<?php
/**
 * Basic application controller that handles the common presentation.
 */
class CAppModel extends CModel {

    protected $_sLanguage = "es";

    /**
     * Validate if $p_mValue is not empty
     *
     * @param <string> $p_sItem = name of field
     * @param <int> $p_mValue = value of field
     * @param <array> $p_someData = some data for function like "message" or "max"
     * @return boolean
     */
    public function valid_not_empty( $p_sItem , $p_mValue , $p_someData )
    {
        $bError = false;
        //it will delete blank spaces
        $p_mValue = trim($p_mValue);
        $p_mValue=trim($p_mValue);
        if ($p_mValue == "") {
            $bError = true;
            $this->_someErrors[$p_sItem] = $p_someData["message"];
        }
        return $bError;
    }

    /**
     * Validate if $p_mValue is double
     *
     * @param <string> $p_sItem = name of field
     * @param <int> $p_mValue = value of field
     * @param <array> $p_someData = some data for function like "message" or "max"
     * @return boolean
     */
    public function valid_double( $p_sItem , $p_mValue , $p_someData )
    {
        $bError = false;
        if (!is_double($p_mValue)) {
            $bError = true;
            $this->_someErrors[$p_sItem] = $p_someData["message"];
        }
        return $bError;
    }

    /**
     * Validate if $p_mValue is a number
     *
     * @param <string> $p_sItem = name of field
     * @param <int> $p_mValue = value of field
     * @param <array> $p_someData = some data for function like "message" or "max"
     * @return boolean
     */
    public function valid_number( $p_sItem , $p_mValue , $p_someData )
    {
        $bError = false;
        if (!is_numeric($p_mValue)) {
            $bError = true;
            $this->_someErrors[$p_sItem] = $p_someData["message"];
        }
        return $bError;
    }

    /**
     * Validate if $p_mValue is not greater than "max"
     *
     * @param <string> $p_sItem = name of field
     * @param <int> $p_mValue = value of field
     * @param <array> $p_someData = some data for function like "message" or "max"
     * @return boolean
     */
    public function valid_max( $p_sItem , $p_mValue , $p_someData )
    {
        $bError = false;
        if (strlen($p_mValue) >= $p_someData["max"]) {
            $bError = true;
            $this->_someErrors[$p_sItem] = $p_someData["message"];
        }
        return $bError;
    }

    /**
     * Validate if $p_mValue is not less than "min"
     *
     * @param <string> $p_sItem = name of field
     * @param <int> $p_mValue = value of field
     * @param <array> $p_someData = some data for function like "message" or "max"
     * @return boolean
     */
    public function valid_min( $p_sItem , $p_mValue , $p_someData )
    {
        $bError = false;
        if (strlen($p_mValue) < $p_someData["min"]) {
            $bError = true;
            $this->_someErrors[$p_sItem] = $p_someData["message"];
        }
        return $bError;
    }

    /**
     * Validate if $p_mValue is not equal than "not_equal"
     *
     * @param <string> $p_sItem = name of field
     * @param <int> $p_mValue = value of field
     * @param <array> $p_someData = some data for function like "message" or "max"
     * @return boolean
     */
    public function valid_not_equal( $p_sItem , $p_mValue , $p_someData )
    {
        $bError = false;
        if ($p_mValue == $p_someData["not_equal"]) {
            $bError = true;
            $this->_someErrors[$p_sItem] = $p_someData["message"];
        }
        return $bError;
    }

    /**
     * Validate if $p_mValue contains a "string"
     *
     * @param <string> $p_sItem = name of field
     * @param <int> $p_mValue = value of field
     * @param <array> $p_someData = some data for function like "message" or "max"
     * @return boolean
     */
    public function valid_string( $p_sItem , $p_mValue , $p_someData )
    {
        $bError = false;
        if (preg_match( $p_mValue , $p_someData["string"] ) == 1 ) {
            $bError = true;
            $this->_someErrors[$p_sItem] = $p_someData["message"];
        }
        return $bError;
    }
    
    public function getUserIdByAgendaId( $p_iAgendaID = null )
    {
        $iUserID = 0;
        try{
            if(is_null( $p_iAgendaID ))
                return $iUserID;

            $someSelectValues = array(
                "p_sColumns" => "users_id",
                "p_sTable" => "agendas",
                "p_sWhere" => "id = ".$p_iAgendaID." AND deleted = 0"
            );

            $this->select($someSelectValues);

            $someResults = $this->get_someData();

            if( count( $someResults ) > 0 )
                return $someResults[0]['users_id'];
        }
        catch(CException $e) {
            throw $e;
        }
        return $iUserID;
    }

}
?>