<?php
/**
 * Description of CUser
 * This class manages the access to the table users
 *
 
 */
class CUser extends CAppModel
{
    // --- ATTRIBUTES ---
    // --- OPERATIONS ---

    /**
     * Constructor for CUser
     */
    public function CUser() {
        try {
            $this->_sTable = 'users';

            $this->set_someValidators(
            array(
                    "dtlbook" => array(
                        "document" => array(
                            C_VALID_NOT_EMPTY => array(
                                "message" => __("The document identification can't be empty",$this->_sLanguage, true)
                            ),
                            "existsDocumentByDocument" => array(
                                "message" => __("This document exists in the database, please login as user.",$this->_sLanguage, true)
                            )
                        ),
                        "cellphone" => array(
                            C_VALID_NOT_EMPTY => array(
                                "message" => __("The phone number can't be empty",$this->_sLanguage, true)
                            ),
                            C_VALID_NUMBER => array(
                                "message" => __("This phone is not a number",$this->_sLanguage, true)
                            )
                        ),
                        "email" => array(
                            C_VALID_NOT_EMPTY => array(
                                "message" => __("The email can't be empty",$this->_sLanguage, true)
                            ),
                            "validEmailString" => array(
                                "message" => __("The email must be valid",$this->_sLanguage, true)
                            )
                        ),
                        "name" => array(
                            C_VALID_NOT_EMPTY => array(
                                "message" => __("The name can't be empty",$this->_sLanguage, true)
                            )
                        ),
                        "cellphonecode" => array(
                            C_VALID_NOT_EMPTY => array(
                                "message" => __("This code can't be empty",$this->_sLanguage, true)
                            )
                        )
                    ),
                    "trial" => array(
                        "cellphone" => array(
                            "existsCellPhoneByPhoneProf" => array(
                                "message" => __("The cellphone provided already exists",$this->_sLanguage, true)
                            ),
                            C_VALID_NOT_EMPTY => array(
                                "message" => __("The phone number can't be empty",$this->_sLanguage, true)
                            ),
                            C_VALID_NUMBER => array(
                                "message" => __("This phone is not a number",$this->_sLanguage, true)
                            )
                        ),
                        "email" => array(
                            "checkExistsEmail" => array(
                                "message" => __("The email provided already exists",$this->_sLanguage, true)
                            ),
                            "validEmailString" => array(
                                "message" => __("The email is not valid",$this->_sLanguage, true)
                            ),
                            C_VALID_NOT_EMPTY => array(
                                "message" => __("The email can't be empty",$this->_sLanguage, true)
                            )
                        ),
                        "name" => array(
                            C_VALID_NOT_EMPTY => array(
                                "message" => __("The name can't be empty",$this->_sLanguage, true)
                            )
                        )
                    ),
                    "add_user" => array(
                        "name" => array(
                            C_VALID_NOT_EMPTY => array(
                                "message" => __("The name can't be empty",$this->_sLanguage, true)
                            )
                        ),
                        "email" => array(
                            "checkExistsEmail" => array(
                                "message" => __("The email provided already exists",$this->_sLanguage, true)
                            ),
                            "validEmailString" => array(
                                "message" => __("The email is not valid",$this->_sLanguage, true)
                            ),
                            C_VALID_NOT_EMPTY => array(
                                "message" => __("The email can't be empty",$this->_sLanguage, true)
                            )
                        )
                    ),
                    "edit_user" => array(
                        "name" => array(
                            C_VALID_NOT_EMPTY => array(
                                "message" => __("The name can't be empty",$this->_sLanguage, true)
                            )
                        ),
                        "email" => array(
                            "validEmailString" => array(
                                "message" => __("The email is not valid",$this->_sLanguage, true)
                            ),
                            C_VALID_NOT_EMPTY => array(
                                "message" => __("The email can't be empty",$this->_sLanguage, true)
                            )
                        )
                    )

                )
            );
        }
        catch(CException $e) {
            throw $e;
        }
    }

    /**
     * This function gets an array of users identified by id
     * @param <type> $p_iIdUser the identifier of the user that we want to retrieve
     * @return <type> returns an array with the user
     */
    public function get_someUsersById($p_iIdUser) {
        try {
            $someData = array();

            //we get the apikey from the database with the public key
            $iReturnValue = $this->select(array("p_sColumns" => "*",
                "p_sWhere" => "id = $p_iIdUser AND deleted=0",
                ));

            if ($iReturnValue == C_WARNING_SELECTEDOK) {
                //we get the data of the executed query
                $someData = $this->get_someData();
            }
            else {
                return false;
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return $someData;
    }
    
    /**
     * This function gets an array of users identified by id
     * @param <type> $p_iIdUser the identifier of the user that we want to retrieve
     * @return <type> returns an array with the user
     */
    public function get_someUsersByIdWithoutDelete($p_iIdUser) {
        try {
            $someData = array();

            //we get the apikey from the database with the public key
            $iReturnValue = $this->select(array("p_sColumns" => "*",
                "p_sWhere" => "id = $p_iIdUser",
                ));

            if ($iReturnValue == C_WARNING_SELECTEDOK) {
                //we get the data of the executed query
                $someData = $this->get_someData();
            }
            else {
                return false;
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return $someData;
    }

    public function get_someAgendasAccessByUserId($p_iUserID) {
        try {
            $someData = array();

            $iReturnValue = $this->select(array(
                "p_sColumns" => "ua.*, a.name",
                "p_sTable" => "users_agendas ua LEFT JOIN agendas a ON a.id = ua.agendas_id",
                "p_sWhere" => "ua.users_id = $p_iUserID AND ua.deleted=0 AND ua.access = 1",
                "p_sOrder" => "a.name"
                ));

            if ($iReturnValue == C_WARNING_SELECTEDOK) {
                //we get the data of the executed query
                $someData = $this->get_someData();
            }
            else {
                return false;
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return $someData;
    }

    public function get_someAgendaByUserIdAndAgendaId($p_iUserId, $p_iAgendaId) {
        try {
            $someData = array();

            $iReturnValue = $this->select(array(
                "p_sColumns" => "ua.*",
                "p_sTable" => "users_agendas ua",
                "p_sWhere" => "ua.users_id = $p_iUserId AND ua.agendas_id = $p_iAgendaId AND ua.deleted=0"
                ));

            if ($iReturnValue == C_WARNING_SELECTEDOK) {
                //we get the data of the executed query
                $someData = $this->get_someData();
            }
            else {
                return false;
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return $someData;
    }

    /**
     * This function gets an array of users identified by id
     * @param <type> $p_iIdUser the identifier of the user that we want to retrieve
     * @return <type> returns an array with the user
     */
    public function get_someUsersByIdWithDeletedOn($p_iIdUser) {
        try {
            $someData = array();

            //we get the apikey from the database with the public key
            $iReturnValue = $this->select(array("p_sColumns" => "*",
                "p_sWhere" => "id = $p_iIdUser ",
                ));

            if ($iReturnValue == C_WARNING_SELECTEDOK) {
                //we get the data of the executed query
                $someData = $this->get_someData();
            }
            else {
                return false;
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return $someData;
    }

    /**
     * This function gets an array of users by timezone id
     * @param <type> $p_iTimezoneId the timezone id
     * @return <type> returns an array with the user
     */
    public function getSomeUsersByTimezone($p_iTimezoneId, $p_iUserType) {
        try {
            $someData = array();

            $iReturnValue = $this->select(
                                        array(
                                            "p_sColumns" => "*",
                                            "p_sWhere" => "deleted = 0 AND timezones_id = $p_iTimezoneId AND type = $p_iUserType",
                                            )
                                        );

            if ($iReturnValue == C_WARNING_SELECTEDOK) {
                $someData = $this->get_someData();
            }
            else {
                return false;
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return $someData;
    }
    
    public function get_someUsersByEmail($p_sEmail, $p_iType=C_USER_USER, $p_iIdCompany = -1) {
        try {
            $someData = array();
            
            $iReturnValue = C_ERROR_EXECSQL;
            
            if ($p_iIdCompany == -1)
            {
                //we get the apikey from the database with the public key
                $iReturnValue = $this->select(array("p_sColumns" => "*",
                "p_sWhere" => "email = '$p_sEmail' and type=".$p_iType." AND deleted=0",
                ));                
            }
            else
            {
                //we get the apikey from the database with the public key
                $iReturnValue = $this->select(array("p_sColumns" => "*",
                "p_sWhere" => "id IN (SELECT c.users_id FROM clients c WHERE c.client_of = $p_iIdCompany) 
                    AND email = '$p_sEmail' and type=".$p_iType." AND deleted=0",
                ));                
            }
            
            if ($iReturnValue == C_WARNING_SELECTEDOK) {
                //we get the data of the executed query
                $someData = $this->get_someData();
            }
            else {
                return false;
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return $someData;
    }

    public function get_someUsersByCellPhone($p_sCellPhone, $p_iType=C_USER_USER, $p_iIdCompany = -1) {
        try {
            $someData = array();
            
            $iReturnValue = C_ERROR_EXECSQL;
            if ($p_iIdCompany == -1)
            {
                //we get the apikey from the database with the public key
                $iReturnValue = $this->select(array("p_sColumns" => "*",
                    "p_sWhere" => "cellphone = '$p_sCellPhone' and type=".$p_iType." AND deleted=0",
                    ));
            }
            else
            {
                $iReturnValue = $this->select(array("p_sColumns" => "*",
                    "p_sWhere" => "id IN (SELECT c.users_id FROM clients c WHERE c.client_of = $p_iIdCompany) 
                    AND cellphone = '$p_sCellPhone' and type=".$p_iType." AND deleted=0",
                    ));
            }

            if ($iReturnValue == C_WARNING_SELECTEDOK) {
                //we get the data of the executed query
                $someData = $this->get_someData();
            }
            else {
                return false;
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return $someData;
    }
    
    public function get_someUsersByPhone($p_sPhone, $p_iType=C_USER_USER, $p_iIdCompany = -1) {
        try {
            $someData = array();
            
            $iReturnValue = C_ERROR_EXECSQL;
            if ($p_iIdCompany == -1)
            {
                //we get the apikey from the database with the public key
                $iReturnValue = $this->select(array("p_sColumns" => "*",
                    "p_sWhere" => "phone = '$p_sPhone' and type=".$p_iType." AND deleted=0",
                    ));
            }
            else
            {
                //we get the apikey from the database with the public key
                $iReturnValue = $this->select(array("p_sColumns" => "*",
                    "p_sWhere" => "id IN (SELECT c.users_id FROM clients c WHERE c.client_of = $p_iIdCompany) AND phone = '$p_sPhone' and type=".$p_iType." AND deleted=0",
                    ));
            }

            if ($iReturnValue == C_WARNING_SELECTEDOK) {
                //we get the data of the executed query
                $someData = $this->get_someData();
            }
            else {
                return false;
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return $someData;
    }

    public function get_someUsersByPhoneUnconfirmed($p_sPhone, $p_iType=C_USER_USER, $p_iIdCompany = -1) {
        try {
            $someData = array();

            $iReturnValue = C_ERROR_EXECSQL;

            //we get the apikey from the database with the public key
            $iReturnValue = $this->select(array(
                                                "p_sColumns" => "*",
                                                "p_sWhere" => "cellphone = '$p_sPhone' and type=".$p_iType." AND deleted=1",
                                                "p_sOrder" => "id ASC"
                ));

            if ($iReturnValue == C_WARNING_SELECTEDOK) {
                //we get the data of the executed query
                $someData = $this->get_someData();

                if(count($someData) > 1){
                    return $someData[count($someData)-1];
                }
            }
            else {
                return false;
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return $someData;
    }

    public function get_someUsersByDocument($p_sDocument, $p_iType=C_USER_USER) {
        try {
            $someData = array();

            //we get the apikey from the database with the public key
            $iReturnValue = $this->select(array("p_sColumns" => "*",
                                                "p_sWhere" => "document = '$p_sDocument' and type=".$p_iType." AND deleted=0",
                ));

            if ($iReturnValue == C_WARNING_SELECTEDOK) {
                //we get the data of the executed query
                $someData = $this->get_someData();
            }
            else {
                return false;
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return $someData;
    }

    public function get_someUsersNotConfirmedByEmail($p_sEmail, $p_iType=C_USER_USER) {
        try {
            $someData = array();

            //we get the apikey from the database with the public key
            $iReturnValue = $this->select(array("p_sColumns" => "*",
                "p_sWhere" => "email = '$p_sEmail' and type=".$p_iType." AND deleted = 1",
                "p_sOrder" => "id ASC",
                ));

            if ($iReturnValue == C_WARNING_SELECTEDOK) {
                //we get the data of the executed query
                $someData = $this->get_someData();

                if(count($someData) > 1){
                    $someTemp = array();
                    $someTemp[0] = $someData[count($someData)-1];                    
                    $someData = $someTemp;
                }
            }
            else {
                return false;
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return $someData;
    }
    
    public function get_someUsersNotConfirmedByEmailAndDocument($p_sEmail , $p_sDocument , $p_iType=C_USER_USER) {
        try {
            $someData = array();

            //we get the apikey from the database with the public key
            $iReturnValue = $this->select(array("p_sColumns" => "*",
                                                "p_sWhere" => "email = '$p_sEmail' AND document = '$p_sDocument' AND type=".$p_iType." AND deleted = 1",
                                               ));

            if ($iReturnValue == C_WARNING_SELECTEDOK) {
                //we get the data of the executed query
                $someData = $this->get_someData();
            }
            else {
                return false;
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return $someData;
    }

    /**
     * This function gets an array of users identified by id
     * @param <type> $p_iIdUser the identifier of the user that we want to retrieve
     * @return <type> returns an array with the user
     */
    public function get_someUsersByAgendaId($p_iAgendaID) {
        try {
            $someData = array();

            //we get the apikey from the database with the public key
            $iReturnValue = $this->select(array(
                "p_sColumns" => "u.* ",
                "p_sTable" => "agendas as a left join users as u on u.id = a.users_id",
                "p_sWhere" => "a.id = $p_iAgendaID AND u.deleted=0 AND a.deleted = 0",
                ));

            if ($iReturnValue == C_WARNING_SELECTEDOK) {
                //we get the data of the executed query
                $someData = $this->get_someData();
            }
            else {
                return false;
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return $someData;
    }

    public function get_someUsersByGMT($p_someGMTs) {
        try {
            $someData = array();

            $sInQuery = implode("','", $p_someGMTs);

            $sInQuery = "'".$sInQuery."'";

            //we get the apikey from the database with the public key
            $iReturnValue = $this->select(
                    array(
                        "p_sColumns" => "u.* ",
                        "p_sTable" => "users u LEFT JOIN regions r ON r.id = u.regions_id",
                        "p_sWhere" => "u.type = ".C_PROF_USER." AND u.parent_users_id = -1 AND u.deleted=0 AND r.gmt IN (".$sInQuery.")",
                    )
                );

            if ($iReturnValue == C_WARNING_SELECTEDOK) {
                //we get the data of the executed query
                $someData = $this->get_someData();
            }
            else {
                return false;
            }
        }
        catch(CException $e) {
            throw $e;
        }
        return $someData;
    }

    public function add_oUserFromWidgetDtl($p_someUsers) {
        try
        {
            $iReturnValue = -1;
            $p_someUsers["users"]["type"] = C_USER_USER;
            $p_someUsers["users"]["cities_id"] = C_CITY_DEFAULT;
            $p_someUsers["users"]["countries_id"] = C_COUNTRY_DEFAULT;
            $p_someUsers["users"]["languages_id"] = 7;
            $p_someUsers["users"]["regions_id"] = C_REGION_DEFAULT;
            $p_someUsers["users"]["credits"] = 0;
            $p_someUsers["users"]["description_tags"] = "";
            $p_someUsers["users"]["name_tags"] = "";
            $p_someUsers["users"]["tags"] = "";
            $p_someUsers["users"]["description"] = "";
            $p_someUsers["users"]["nick"] = "";
            $p_someUsers["users"]["deleted"] = 1;
            $p_someUsers["users"]["created"] = date("Y-m-d H:i:s",mktime());


            $iReturnValue = $this->insert($p_someUsers,"users",true);
            if ($iReturnValue == C_WARNING_INSERTEDOK)
                $iReturnValue = $this->get_iIdLastInserted();
            else
                $iReturnValue = -1;

        }
        catch(CException $e)
        {
            throw $e;
        }

        return $iReturnValue;
    }

    public function add_oUser($p_someUsers) {
        try {
            //first we insert the user
            $someToInsert["users"] = $p_someUsers;            

            $iReturnValue = $this->insert($p_someUsers,"users",false);
            if ($iReturnValue == C_WARNING_INSERTEDOK)
                $iReturnValue = $this->get_iIdLastInserted();
            else
            {
                return -1;

            }

        }
        catch(CException $e) {
            $this->rollBackTransaction();
            throw $e;
        }

        return $iReturnValue;
    }

    public function add_oUser_validates($p_someUsers, $p_someAgendasIds) {
        try {
            $this->startTransaction();

            $iReturnValue = $this->insert($p_someUsers,true,true);
            
            if ($iReturnValue == C_WARNING_INSERTEDOK){
                $iReturnValue = $this->get_iIdLastInserted();

                $someAgendas = array();

                foreach ($p_someAgendasIds as $key => $agendasId) {
                    $someAgendas["users_agendas"][$key]["users_id"] = $iReturnValue;
                    $someAgendas["users_agendas"][$key]["agendas_id"] = $key;
                }

                if(count($someAgendas) > 0){
                    if($this->insert($someAgendas,false,false) == C_WARNING_INSERTEDOK){
                        $this->endTransaction();
                        return $iReturnValue;
                    }
                    else{
                        $this->rollBackTransaction();
                        return -1;
                    }
                }
                else{
                    $this->endTransaction();
                    return $iReturnValue;
                }
            }
            else{
                $this->rollBackTransaction();
                return -1;
            }
        }
        catch(CException $e) {
            $this->rollBackTransaction();
            throw $e;
        }        
    }

    public function edit_oUser_validates($p_iUserID, $p_someUsers, $p_someAgendasIds) {
        try {
            $this->startTransaction();

            $sWhere = "id = $p_iUserID AND deleted = 0";
            $iReturnValue = $this->update($p_someUsers,$sWhere);

            if ($iReturnValue == C_WARNING_UPDATEDOK){
                $iReturnValue = $this->get_iIdLastInserted();

                $someAgendas = $this->get_someAgendasAccessByUserId($p_iUserID);

                $someAgendasToUpdate = array();

                $iCount = 0;

                foreach ($someAgendas as $agenda) {
                    if(isset ($p_someAgendasIds[$agenda["agendas_id"]]) ){
                        unset($someAgendas[$iCount]);
                        unset($p_someAgendasIds[$agenda["agendas_id"]]);
                    }
                    $iCount++;
                }

                $someAgendasIdToDeleteAccess = $someAgendas;
                $someAgendasIdToInsert = $p_someAgendasIds;

                if(count($someAgendasIdToDeleteAccess) > 0 || count($someAgendasIdToInsert) > 0 ){
                    $someAgendas = array();
                    $someAgendas["users_agendas"]["access"] = 0;

                    foreach ($someAgendasIdToDeleteAccess as $agendaToDeleteAcess) {
                        $sWhere = "users_id = $p_iUserID AND deleted = 0 AND agendas_id = ".$agendaToDeleteAcess["agendas_id"];
                        
                        if( $this->update($someAgendas,$sWhere,false) != C_WARNING_UPDATEDOK){
                            $this->rollBackTransaction();
                            return -1;
                        }
                    }

                    $someAgendas = array();

                    if(count($someAgendasIdToInsert) > 0){
                        $someAgendas["users_agendas"]["access"] = 1;

                        foreach ($someAgendasIdToInsert as $agendaToUpdateAcess) {
                            $someAgendasAccess = $this->get_someAgendaByUserIdAndAgendaId($p_iUserID, $agendaToUpdateAcess);

                            if(count($someAgendasAccess) > 0){
                                $sWhere = "users_id = $p_iUserID AND deleted = 0 AND agendas_id = ".$agendaToUpdateAcess;

                                if( $this->update($someAgendas,$sWhere,false) == C_WARNING_UPDATEDOK){
                                    unset($someAgendasIdToInsert[$agendaToUpdateAcess]);
                                }
                                else{
                                    $this->rollBackTransaction();
                                    return -1;
                                }
                            }
                        }

                        $someAgendas = array();

                        if(count($someAgendasIdToInsert) > 0){
                            foreach ($someAgendasIdToInsert as $key => $agendasId) {
                                $someAgendas["users_agendas"][$key]["users_id"] = $p_iUserID;
                                $someAgendas["users_agendas"][$key]["agendas_id"] = $key;
                            }

                            if( $this->insert($someAgendas,false,false) != C_WARNING_INSERTEDOK){
                                $this->rollBackTransaction();
                                return -1;
                            }
                        }
                    }
                }
            }
            else{
                $this->rollBackTransaction();
                return -1;
            }

            $this->endTransaction();
            return C_WARNING_UPDATEDOK;
        }
        catch(CException $e) {
            $this->rollBackTransaction();
            throw $e;
        }
    }

    /**
     * This function updates the cell phone code
     * @param <type> $p_someUsers the parametters getted from the view, in this case cellphonecode
     * @param <type> $p_iIdUser the identifier of the user that it will update
     * @return <type> if there is error it returns -1, else it returns the id of the user
     */
    public function edit_oUserFromWidgetDtl($p_someUpdate, $p_iIdUser) {
        try
        {
            $p_someUpdateAux["users"]["deleted"] = 0;
            $p_someUpdateAux["users"]["updated"] = date("Y-m-d H:i:s",mktime());
            $p_someUpdateAux["users"]["password"] = md5($p_someUpdate["users"]["cellphonecode"]);
            $p_someUpdateAux["users"]["cellphonecode"] = $p_someUpdate["users"]["cellphonecode"];


            $iReturnValue = $this->update($p_someUpdateAux, "id = $p_iIdUser and deleted = 1");
            if ($iReturnValue == C_WARNING_UPDATEDOK)
                $iReturnValue = $p_iIdUser;
            else
                $iReturnValue = -1;

        }
        catch(CException $e) {
            throw $e;
        }

        return $iReturnValue;
    }

    /**
     * This function updates the cell phone code
     * @param <type> $p_someUsers the parametters getted from the view, in this case cellphonecode
     * @param <type> $p_iIdUser the identifier of the user that it will update
     * @return <type> if there is error it returns -1, else it returns the id of the user
     */
    public function confirmPhoneUser($p_sPhone, $p_iIdUser) {
        try
        {
            $someUpdate["users"]["deleted"] = 0;
            $someUpdate["users"]["cellphone"] = $p_sPhone;
            $someUpdate["users"]["updated"] = date("Y-m-d H:i:s",mktime());

            $iReturnValue = $this->update($someUpdate, "id = $p_iIdUser and deleted = 1");
            if ($iReturnValue == C_WARNING_UPDATEDOK)
                $iReturnValue = $p_iIdUser;
            else
                $iReturnValue = -1;

        }
        catch(CException $e) {
            throw $e;
        }

        return $iReturnValue;
    }

    /**
     * This function updates the cell phone code
     * @param <type> $p_someUsers the parametters getted from the view, in this case cellphonecode
     * @param <type> $p_iIdUser the identifier of the user that it will update
     * @return <type> if there is error it returns -1, else it returns the id of the user
     */
    public function changePhoneUser($p_sPhone, $p_iValidateKey , $p_iIdUser) {
        try
        {
            $someUpdate["users"]["cellphone"] = $p_sPhone;
            $someUpdate["users"]["cellphonecode"] = $p_iValidateKey;
            $someUpdate["users"]["updated"] = date("Y-m-d H:i:s",mktime());

            $iReturnValue = $this->update($someUpdate, "id = $p_iIdUser and deleted = 1");
            if ($iReturnValue == C_WARNING_UPDATEDOK)
            {
                $iReturnValue = $p_iIdUser;
            }
            else
            {
                $iReturnValue = -1;
            }

        }
        catch(CException $e) {
            throw $e;
        }

        return $iReturnValue;
    }

    /**
     * This function is a custom VALIDATOR for cell phone.
     * @param <string> $p_sItem = name of the field defined in the validators, in this case phone
     * @param <int> $p_mValue = the value written by the user
     * @param <array>"min" => 4,
                     "message" => "The name have to be greater than 4", min or message will depend of your definition.
     * @return <type> if exists it will return true else it will return false
     */
    public function existsCellPhoneByPhone($p_sItem , $p_mValue , $p_someData) {
        try {
            $bReturnValue = false;
            if (trim($p_mValue) != "") {
                $iReturnValue = $this->select(array("p_sColumns" => "*",
                                  "p_sWhere" => "cellphone = '$p_mValue' AND deleted=0 and type=".C_USER_USER
                                 ));
                if ($iReturnValue == C_WARNING_SELECTEDOK) {
                    //we get the data of the executed query
                    $someData = $this->get_someData();
                    if (count($someData)>0) {
                        $bReturnValue = true;
                        $this->_someErrors[$p_sItem] = $p_someData["message"];
                    }
                }
                else {
                    $bReturnValue = false;
                }
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return $bReturnValue;
    }

    public function existsCellPhoneByPhoneProf($p_sItem , $p_mValue , $p_someData) {
        try {
            $bReturnValue = false;
            if (trim($p_mValue) != "") {
                $iReturnValue = $this->select(array("p_sColumns" => "*",
                                  "p_sWhere" => "cellphone = '$p_mValue' AND deleted=0 and type=".C_PROF_USER
                                 ));
                if ($iReturnValue == C_WARNING_SELECTEDOK) {
                    //we get the data of the executed query
                    $someData = $this->get_someData();
                    if (count($someData)>0) {
                        $bReturnValue = true;
                        $this->_someErrors[$p_sItem] = $p_someData["message"];
                    }
                }
                else {
                    $bReturnValue = false;
                }
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return $bReturnValue;
    }

    public function existsDocumentByDocument($p_sItem , $p_mValue , $p_someData) {
        try {
            $bReturnValue = false;
            if (trim($p_mValue) != "") {
                $iReturnValue = $this->select(array("p_sColumns" => "*",
                                                    "p_sWhere" => "document = '$p_mValue' AND deleted=0 and type=".C_USER_USER
                                                   ));
                if ($iReturnValue == C_WARNING_SELECTEDOK) {
                    //we get the data of the executed query
                    $someData = $this->get_someData();
                    if (count($someData)>0) {
                        $bReturnValue = true;
                        $this->_someErrors[$p_sItem] = $p_someData["message"];
                    }
                }
                else {
                    $bReturnValue = false;
                }
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return $bReturnValue;
    }

    public function existsEmailByEmail($p_sItem , $p_mValue , $p_someData) {
        try {
            $bReturnValue = false;
            if (trim($p_mValue) != "") {
                $iReturnValue = $this->select(array("p_sColumns" => "*",
                                  "p_sWhere" => "email = '$p_mValue' AND deleted=0 and type=".C_USER_USER
                                 ));
                if ($iReturnValue == C_WARNING_SELECTEDOK) {
                    //we get the data of the executed query
                    $someData = $this->get_someData();
                    if (count($someData)>0) {
                        $bReturnValue = true;
                        $this->_someErrors[$p_sItem] = $p_someData["message"];
                    }
                }
                else {
                    $bReturnValue = false;
                }
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return $bReturnValue;
    }

    public function validEmailString($p_sItem , $p_mValue , $p_someData) {
        try {
            $bReturnValue = true;
            if (trim($p_mValue) != "") {
                if ((substr_count($p_mValue,"@") == 1) && (substr($p_mValue,0,1) != "@") && (substr($p_mValue,strlen($p_mValue)-1,1) != "@")){
                    if (substr_count($p_mValue,".")>= 1){
                        $term_dom = substr(strrchr ($p_mValue, '.'),1);
                        if (strlen($term_dom) >= 1 && (!strstr($term_dom,"@")) ){
                            $bReturnValue = false;
                        }
                    }
                }
            }

            if($bReturnValue == true){
                $this->_someErrors[$p_sItem] = $p_someData["message"];
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return $bReturnValue;
    }

    public function checkExistsEmail($p_sItem , $p_mValue , $p_someData) {
        try {
            $bReturnValue = true;
            $someProfUserByEmail = $this->get_someUsersByEmail($p_mValue, C_PROF_USER);

            if(count($someProfUserByEmail) > 0){
                $this->_someErrors[$p_sItem] = $p_someData["message"];
            }
            else{
                $bReturnValue = false;
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return $bReturnValue;
    }

    public function validateByFieldDefaultEmail($p_someData, $p_iIdCompany, $p_sField = "email") {
        try {
            $bReturnValue = false;
            $iReturnValue = $this->select(array("p_sColumns" => "*",
                                  "p_sWhere" => "id IN (SELECT c.users_id FROM clients c WHERE c.client_of = $p_iIdCompany) AND $p_sField = '".$p_someData["users"]["emailorphone"]."'
                                      AND password = '".md5($p_someData["users"]["password"])."' AND deleted=0 AND type=".C_USER_USER
                                 ));
            if ($iReturnValue == C_WARNING_SELECTEDOK) {
                 $someData = $this->get_someData();
                 if (count($someData) > 0) {
                    $bReturnValue = $someData[0]["id"];
                 }
                 else {
                     $bReturnValue = false;
                 }
            }
            else {
                $bReturnValue = false;
            }
        }
        catch(CException $e) {
            throw $e;
        }
        return $bReturnValue;
    }

    public function validateByFieldDefaultDocument($p_someData) {
        try {
            $bReturnValue = false;
            $iReturnValue = $this->select(array("p_sColumns" => "*",
                                                "p_sWhere" => "document = '".$p_someData["users"]["document"]."'
                                                AND password = '".md5($p_someData["users"]["password"])."' AND deleted=0 AND type=".C_USER_USER
                                               ));
            if ($iReturnValue == C_WARNING_SELECTEDOK) {
                 $someData = $this->get_someData();
                 if (count($someData) > 0) {
                    $bReturnValue = $someData[0]["id"];
                 }
                 else {
                     $bReturnValue = false;
                 }
            }
            else {
                $bReturnValue = false;
            }
        }
        catch(CException $e) {
            throw $e;
        }
        return $bReturnValue;
    }
    
    /**
     * This function validates a user by email or phone for an specified company
     * @param type $p_someData the fields that have the values to be updated
     * @param type $p_iIdClientOf the id of the company where the user wants to access
     * @param type $p_sField the field for the validation. Normally email or phone
     * @return boolean if the user doesn't exists it returns false, otherwise it returns the id of the user
     */
    public function validateByFieldDefaultEmailTempPass($p_someData, $p_iIdCompany, $p_sField = "email") {
        try {
            $bReturnValue = false;
            $iReturnValue = $this->select(array("p_sColumns" => "*",
                                  "p_sWhere" => "id IN (SELECT c.users_id FROM clients c WHERE c.client_of = $p_iIdCompany) AND $p_sField = '".$p_someData["users"]["emailorphone"]."'
                                      AND pass_temp = '".md5($p_someData["users"]["password"])."' AND deleted=0 AND type=".C_USER_USER
                                 ));
            if ($iReturnValue == C_WARNING_SELECTEDOK) {
                 $someData = $this->get_someData();
                 if (count($someData) > 0) {
                    $bReturnValue = $someData[0]["id"];
                 }
                 else {
                     $bReturnValue = false;
                 }
            }
            else {
                $bReturnValue = false;
            }
        }
        catch(CException $e) {
            throw $e;
        }
        return $bReturnValue;
    }

    public function validateByFieldDocumentTempPass($p_someData) {
        try {
            $bReturnValue = false;
            $iReturnValue = $this->select(array("p_sColumns" => "*",
                                                "p_sWhere" => "document = '".$p_someData["users"]["document"]."'
                                                AND pass_temp = '".md5($p_someData["users"]["password"])."' AND deleted=0 AND type=".C_USER_USER
                                               ));
            if ($iReturnValue == C_WARNING_SELECTEDOK) {
                 $someData = $this->get_someData();
                 if (count($someData) > 0) {
                    $bReturnValue = $someData[0]["id"];
                 }
                 else {
                     $bReturnValue = false;
                 }
            }
            else {
                $bReturnValue = false;
            }
        }
        catch(CException $e) {
            throw $e;
        }
        return $bReturnValue;
    }

    public function getAvailableSMS( $p_iUserId){
        $someResults= null;

        try{
            $someSelectValues = array(
                "p_sColumns" => "u.cache_messages",
                "p_sTable" => "users u",
                "p_sWhere" => "u.id = ".$p_iUserId
            );

            $this->select($someSelectValues);

            $someResults = $this->get_someData();

        }
        catch(CException $e) {
            throw $e;
        }
        return $someResults;
    }

    public function substractSMS($p_oUser, $p_iQuantity){
        try
        {
            $iTotal = $p_oUser["cache_messages"]-$p_iQuantity;
            $iIdUser = $p_oUser["id"];

            $someUpdate["users"]["cache_messages"] = $iTotal;

            $iReturnValue = $this->update($someUpdate, "id = $iIdUser and deleted = 0");
            if ($iReturnValue == C_WARNING_UPDATEDOK)
                $iReturnValue = $iIdUser;
            else
                $iReturnValue = -1;
            
        }
        catch(CException $e) {
            throw $e;
        }

        return $iReturnValue;
    }
    
    public function addSMS($p_oUser, $p_iQuantity){
        try
        {
            $iTotal = $p_oUser["cache_messages"]+$p_iQuantity;
            $iIdUser = $p_oUser["id"];

            $someUpdate["users"]["cache_messages"] = $iTotal;

            $iReturnValue = $this->update($someUpdate, "id = $iIdUser and deleted = 0");
            if ($iReturnValue == C_WARNING_UPDATEDOK)
                $iReturnValue = $iIdUser;
            else
                $iReturnValue = -1;
            
        }
        catch(CException $e) {
            throw $e;
        }

        return $iReturnValue;
    }
  
    public function updatePrefReference($p_iIdUser, $p_sPrefReference) {
        try {

            $someUpdate["users"]["prefreference"] = $p_sPrefReference;

            $iReturnValue = $this->update($someUpdate, "id = $p_iIdUser and deleted = 0",false);
            if ($iReturnValue == C_WARNING_UPDATEDOK)
                $iReturnValue = C_WARNING_UPDATEDOK;
            else
                $iReturnValue = -1;
            }
        catch(CException $e) {
            throw $e;
        }
           return $iReturnValue;
        }

    public function getPrefReference($p_iIdUser) {
             try {
                $someSelectValues = array(
                    "p_sColumns" => "u.prefreference",
                    "p_sTable" => "users u",
                    "p_sWhere" => "u.deleted = 0 and u.id = $p_iIdUser"                    
                    
                );
                //PLEASE, don't touch ORDER AND DIRECTION
                $this->select($someSelectValues);
                $someResults = $this->get_someData();
                if (count($someResults) > 0) {
                    return $someResults[0]["prefreference"];
                }
            }
            catch(CException $e) {
                throw $e;
            }

            return "";
        }

    public function get_someUsersByPhoneOrEmail($p_sClientPhone , $p_sClientEmail , $p_iUserID){
        try {
            $someData = array();
            $iReturnValue = "";

            if(strlen($p_sClientPhone) > 0 && strlen($p_sClientEmail) > 0 ){
                $iReturnValue = $this->select(array("p_sColumns" => "cl.id as id_client, u.*",
                                                "p_sTable" => "users u, clients cl",
                                                "p_sWhere" => " ( u.email = '$p_sClientEmail' OR u.cellphone = '$p_sClientPhone' ) AND u.deleted = 0 AND u.type = " . C_USER_USER . " AND u.id IN (
                                                                        SELECT c.users_id
                                                                        FROM clients c
                                                                        WHERE c.client_of = $p_iUserID AND c.deleted = 0
                                                                 ) GROUP BY u.id",
                                                )
                                         );
            }
            else if(strlen($p_sClientEmail) > 0){
                $iReturnValue = $this->select(array("p_sColumns" => "cl.id as id_client, u.*",
                                                "p_sTable" => "users u, clients cl",
                                                "p_sWhere" => "u.email = '$p_sClientEmail' AND u.deleted = 0 AND u.type = " . C_USER_USER . " AND u.id IN (
                                                                        SELECT c.users_id
                                                                        FROM clients c
                                                                        WHERE c.client_of = $p_iUserID AND c.deleted = 0
                                                                 ) GROUP BY u.id",
                                                )
                                         );
            }
            else if(strlen($p_sClientPhone) > 0){
                $iReturnValue = $this->select(array("p_sColumns" => "cl.id as id_client, u.*",
                                                "p_sTable" => "users u, clients cl",
                                                "p_sWhere" => "u.cellphone = '$p_sClientPhone' AND u.deleted = 0 AND u.type = " . C_USER_USER . " AND u.id IN (
                                                                        SELECT c.users_id
                                                                        FROM clients c
                                                                        WHERE c.client_of = $p_iUserID AND c.deleted = 0
                                                                 ) GROUP BY u.id",
                                                )
                                         );
            }

            if ($iReturnValue == C_WARNING_SELECTEDOK) {
                //we get the data of the executed query
                $someData = $this->get_someData();
            }
            else {
                return false;
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return $someData;
    }

    public function getLanguageCodeByLanguageId($p_iLanguageId) {
        try {
            $someData = array();

            $someSelectValues = array(
                    "p_sColumns" => "l.code",
                    "p_sTable" => "languages l",
                    "p_sWhere" => "l.id = '$p_iLanguageId'",
                );

            $iReturnValue = $this->select($someSelectValues);

            if ($iReturnValue == C_WARNING_SELECTEDOK) {
                $someData = $this->get_someData();
            }
            else {
                return false;
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return $someData;
    }

    public function getLanguageIdByLanguageCode($p_iLanguageCode) {
        try {
            $someData = array();

            $someSelectValues = array(
                    "p_sColumns" => "l.id",
                    "p_sTable" => "languages l",
                    "p_sWhere" => "l.code = '$p_iLanguageCode'",
                );

            $iReturnValue = $this->select($someSelectValues);

            if ($iReturnValue == C_WARNING_SELECTEDOK) {
                $someData = $this->get_someData();
            }
            else {
                return false;
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return $someData;
    }

    public function updateLanguagesId($p_iProfId , $p_sCode) {
        try {
            $iLanguageId = $this->getLanguageIdByLanguageCode($p_sCode);

            if(count($iLanguageId) > 0){
                $someUserData = array();
                $someUserData["users"]["languages_id"] = $iLanguageId[0]["id"];

                $sWhere = 'id = '.$p_iProfId;

                if($this->update($someUserData,$sWhere,false) == C_WARNING_UPDATEDOK){
                    return true;
                }
            }
            else{
                return false;
            }
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function get_firstParentIdByUserId($p_iUserId = null) {
        try {
            $iParentId = -1;

            if($p_iUserId > 0){
                $bFlag = true;
                $iParentId = $p_iUserId;

                while($bFlag){
                    $someUserData = $this->get_someUsersById($iParentId);

                    if($someUserData[0]["parent_users_id"] != -1){
                        $iParentId = $someUserData[0]["parent_users_id"];
                    }
                    else{
                        $bFlag = false;
                    }
                }
                return $iParentId;
            }
            else{
                return $iParentId;
            }
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function get_someUserParentChild($p_iUserLogged, $p_iUserParentId = null){
        try {
            $someChildIds = array();
            $someChildIdsTemp = array();
            $someUsersData = array();

            $someLoggedUserData = $this->get_someUsersById($p_iUserLogged);
            $iParentId = $someLoggedUserData[0]["id"];
//            $iParentId = $someLoggedUserData[0]["parent_users_id"];

            $someUsersData = $this->get_someUsersByParentId($iParentId);

            if(count($someUsersData) > 0){
                foreach ($someUsersData as $userData) {
                    $someChildIds[$userData["id"]] = $userData["id"];
                }

                reset($someChildIds);
                $bFlagNext = true;

                while($bFlagNext != false){
                    $bFlagNext = $someCurrentItem = each($someChildIds);
                    $someUsersData = $this->get_someUsersByParentId($someCurrentItem["key"]);

                    if(count($someUsersData) > 0 && $someUsersData != false){
                        foreach ($someUsersData as $userData) {
                            $someChildIds[$userData["id"]] = $userData["id"];
                        }
                    }
                }
            }

            if($p_iUserLogged == $p_iUserParentId){
                unset ($someChildIds[$p_iUserLogged]);
            }

            return $someChildIds;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function get_someUsersByParentId($p_iUserParent) {
        try {
            $someData = array();

            //we get the apikey from the database with the public key
            $iReturnValue = $this->select(
                                        array(
                                            "p_sColumns" => "*",
                                            "p_sWhere" => "parent_users_id = $p_iUserParent AND deleted=0",
                                        ));

            if ($iReturnValue == C_WARNING_SELECTEDOK) {
                //we get the data of the executed query
                $someData = $this->get_someData();
            }
            else {
                return false;
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return $someData;
    }

    public function get_someUsersProfCellphoneByCountry($p_iCellphone, $p_iCountryId) {
        try {
            $someData = array();

            //we get the apikey from the database with the public key
            $iReturnValue = $this->select(
                                        array(
                                            "p_sColumns" => "*",
                                            "p_sWhere" => "cellphone = $p_iCellphone AND deleted=0 AND type=".C_PROF_USER." AND countries_id=$p_iCountryId",
                                        ));

            if ($iReturnValue == C_WARNING_SELECTEDOK) {
                //we get the data of the executed query
                $someData = $this->get_someData();
            }
            else {
                return false;
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return $someData;
    }

    public function get_someProfUsers() {
        try {
            $someData = array();

            //we get the apikey from the database with the public key
            $iReturnValue = $this->select(
                                        array(
                                            "p_sColumns" => "*",
                                            "p_sWhere" => "type = ".C_PROF_USER." AND deleted = 0 AND parent_users_id = -1",
                                            "p_sOrder" => "name asc"
                                        ));

            if ($iReturnValue == C_WARNING_SELECTEDOK) {
                //we get the data of the executed query
                $someData = $this->get_someData();
            }
            else {
                return false;
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return $someData;
    }

    public function udpateUserAndClientDeleted($p_iClientId) {
        try {
            $this->startTransaction();

            //check client deleted field
            $someClientData = $this->_oController->_oClient->getClientByIdClient($p_iClientId);

            if(count($someClientData) <= 0){
                //client not exit with deleted  = 0, update to this
                $someClientParameters["clients"]["deleted"] = 0;
                
                if($this->_oController->_oClient->update($someClientParameters, "id=".$p_iClientId) == C_WARNING_UPDATEDOK ){
                    $this->endTransaction();
                    return true;
//                    $someClientData = $this->_oController->_oClient->getClientByIdClient($p_iClientId);
//
//                    //check user deleted field
//                    if($someClientData[0]["users_id"] > 0){
//                        $someUserData = $this->get_someUsersById($someClientData[0]["users_id"]);
//
//                        if(count($someUserData) <= 0){
//                            $someUserParameters["users"]["deleted"] = 0;
//                            if($this->update($someUserParameters, "id=".$someClientData[0]["users_id"]) != C_WARNING_UPDATEDOK ){
//                                $this->rollBackTransaction();
//                                return false;
//                            }
//                        }
//                    }
                }
                else{
                    $this->rollBackTransaction();
                    return false;
                }
            }
//            else{
//                //check user deleted field
//                if($someClientData[0]["users_id"] > 0){
//                    $someUserData = $this->get_someUsersById($someClientData[0]["users_id"]);
//
//                    if(count($someUserData) <= 0){
//                        $someUserParameters["users"]["deleted"] = 0;
//                        if($this->update($someUserParameters, "id=".$someClientData[0]["users_id"]) != C_WARNING_UPDATEDOK ){
//                            $this->rollBackTransaction();
//                            return false;
//                        }
//                    }
//                }
//            }
        $this->endTransaction();
        return true;

        }
        catch(CException $e) {
            throw $e;
        }
    }
    
    public function activateDeactivate($p_iIdUser, $p_iState)
    {
        try
        {
             $this->startTransaction();
             
             //first it will deactive the user
             $sWhere = "id = $p_iIdUser";
             
             $someUsers = array();
             $someUsers["users"]["deleted"] = $p_iState;
             
             $iReturnValue = $this->update($someUsers,$sWhere);
             if ($iReturnValue == C_WARNING_UPDATEDOK)
             {
                 $someApis = array();
                 $someApis["apikeys"]["deleted"] = $p_iState;
                 
                 $sWhere = "users_id = $p_iIdUser";
                 $iReturnValue = $this->update($someApis,$sWhere, false);
                 if ($iReturnValue != C_WARNING_UPDATEDOK)
                 {
                     $this->rollBackTransaction();
                     return false;
                 }     
                 $this->endTransaction();
             }
             else
             {
                 $this->rollBackTransaction();
                 return false;
             }
             
        }
        catch(CException $e)
        {
            $this->rollBackTransaction();
            throw $e;
        }
        
        return true;
    }
    
    public function getSomeUsersByProfile($p_iProfileType, $p_iIdAdmin = null, $p_iUserId = null)
    {
        try {
            $someData = array();
            
            if ($p_iProfileType == C_PROFILE_ROOT)
            {
                $someValues = array(
                        'p_sColumns' => 'u.*',
                        "p_sTable" => "users u",
                        "p_sWhere" => "u.type = ".C_PROF_USER." AND u.parent_users_id = -1");
            }
            else
            {
               $someValues = array(
                        'p_sColumns' => 'u.*',
                        "p_sTable" => "users u",
                        "p_sWhere" => "(u.id IN (SELECT au.users_id 
                                                 FROM admins_users as au WHERE au.admins_id = $p_iIdAdmin) 
                                            AND (u.type = ".C_PROF_USER." AND u.parent_users_id = -1 AND u.id = $p_iUserId))");
                
            }
            
            $iReturnValue = $this->select($someValues);   

            if ($iReturnValue == C_WARNING_SELECTEDOK) {
                //we get the data of the executed query
                $someData = $this->get_someData();
            }
            
        }
        catch(CException $e) {
            throw $e;
        }

        return $someData;
    }
    
    
    public function addUserToAdmin($p_iIdAdmin, $p_iIdUser)
    {
        try
        {
            $someToInsert["admins_users"]["admins_id"] = $p_iIdAdmin;            
            $someToInsert["admins_users"]["users_id"] = $p_iIdUser;            

            $iReturnValue = $this->insert($someToInsert,false);
            if ($iReturnValue == C_WARNING_INSERTEDOK)
            {
                $iReturnValue = $this->get_iIdLastInserted();
            }
            else
            {
                $iReturnValue = -1;
            }

        }
        catch(CException $e)
        {
            throw $e;
        }
        return $iReturnValue;
    }
    
    public function deleteUserToAdmin($p_iIdAdmin, $p_iIdUser)
    {
        try
        {            

            $iReturnValue = $this->delete("admins_users","admins_id = $p_iIdAdmin and users_id = $p_iIdUser");
            if ($iReturnValue == C_WARNING_INSERTEDOK)
                $iReturnValue = $this->get_iIdLastInserted();
            else
            {
                $iReturnValue = -1;
            }

        }
        catch(CException $e)
        {
            throw $e;
        }
        return $iReturnValue;
    }
}
?>
