<?php

/**
 * Description of CLoginsController.
 * This controller will control if a controller and its action are in the list
 * of privated zones. In this case we will have to show a validation login window,
 * because a normal user mustn't has access to this section.
 *
 * @author Nubesis S.L. http://www.nubesis.com
 */
class CLoginsController extends CAppController {

    /**
     *
     * @param <type> prueba
     */
    private $_somePrivatedUserZones = array();
    private $_somePrivatedAdminZones = array();
    private $_somePrivatedProfZones = array();

    /**
     * Constructor for CLoginsController
     */
    public function CLoginsController() {
        try {
            parent::CAppcontroller();
            $this->setType(C_PLUGINS);
            $this->_oView = new CAppView("index.php",C_PLUGINS . "logins/views/");            
            $this->setTitle(C_TITLE_PAGE);
            $this->setMetaKeywords(C_METAKEYWORDS);
            $this->setMetaDescription(C_METADESCRIPTION);
            $this->set_somePrivatedAdminZones(eval(C_PRIVATEADMINZONES_SCREENS));
            $this->set_somePrivatedUserZones(eval(C_PRIVATEUSERZONES_SCREENS));
            $this->set_somePrivatedProfZones(eval(C_PRIVATEPROFZONES_SCREENS));
            
            

        }
        catch(CException $e) {
            throw $e;
        }
    }

    /**
     * This function will evaluate a controller and an action passed by parameter.
     * If they belong to a privated zone, we will have to show a validation login window otherwise it will load the correct view.
     * @param <String> $p_sController the string with the name of the controller to test
     * @param <String> $p_sAction the action of the controller that is going to be invoked
     * @return This function returns null if a controller is not in a "Privated Zone" or it returns an Object CView with the
     * new view that CController the system will show to the user.
     */
    public function evaluate($p_sController, $p_sAction) {
        try
        {
            //we have to check if the controller called and its action are in PrivatedAdminZone or PrivatedUserZone
            $oView = null;

            if (isset($this->_somePrivatedAdminZones[$p_sController]))
            {
               
               //if the action controller is * or it's in the array then we show the login window
               $sActions = $this->_somePrivatedAdminZones[$p_sController];
               //if ((strcasecmp($this->_somePrivatedAdminZones[$p_sController],$p_sAction) == 0) ||
               if ((stristr($sActions,$p_sAction) != false) ||
                      (strcasecmp($this->_somePrivatedAdminZones[$p_sController],"*")==0))
               {
                   //it's an admin zone
                   //but we have only to show the login window if you are not logged. For this reason we have to search
                   //in the sessions of the system.
                   $oView = new CAppView("backend.php",C_PLUGINS . "logins/views/");
                   $oView->set_sParentController("logins");
                   return $oView;
               }

            }
            elseif (isset($this->_somePrivatedUserZones[$p_sController])) {
               
               //it's a user zone
               //if the action controller is * or it's in the array then we show the login window
               $sActions = $this->_somePrivatedUserZones[$p_sController];
               //if ((strcasecmp($this->_somePrivatedUserZones[$p_sController],$p_sAction) == 0) ||
               if ((stristr($sActions,$p_sAction) != false) ||
                      (strcasecmp($this->_somePrivatedUserZones[$p_sController],"*")==0))
               {
                    $oView = new CAppView("index.php",C_PLUGINS . "logins/views/");
                    $oView->set_sParentController("logins");
                    return $oView;
               }
            }
            elseif (isset($this->_somePrivatedProfZones[$p_sController])) {
                //it's a professional zone
               //if the action controller is * or it's in the array then we show the login window
                $sActions = $this->_somePrivatedProfZones[$p_sController];
               //if ((strcasecmp($this->_somePrivatedProfZones[$p_sController],$p_sAction) == 0) ||
               if ((stristr($sActions,$p_sAction) != false) ||
                      (strcasecmp($this->_somePrivatedProfZones[$p_sController],"*")==0))
               {
                    $oView = new CAppView("professional.php",C_PLUGINS . "logins/views/");
                    $oView->set_sParentController("logins");
                    return $oView;
               }
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return $oView;
    }

    /**
     *
     * @param <type> $p_oController The controller that we want to evaluate for the access of a user. If it's needed, It will redirect
     * to a concrete page. To have a correct evaluation you will have to define the private zones or areas in app_config file.
     * @return <boolean> This function returns true if the requested controller is correct. If the url where
     * you want to access is not valid or needs a validation page it will returns false;
     */
    public function evaluateAndRedirect(&$p_oController) {
        try {
            //we have to execute this code only if the plugin is installed
            $oSession = new CAppSession();            
            $oView = $p_oController->get_oView();

            //it's in a public zone
            //if(isset($this->_somePublicZones[$oView->get_sParentController()])) return true;

            $bReturnCookie = $this->checkcookies($p_oController);


            //if a user hasn't validated we have to show the validation login window
            if ($oSession->__get("type")==false) {
                
                //this is a special case for admins. If it's and admin user it will access to bookitit.
                if ($oSession->__get("typeadmin") == C_ADMIN_USER)  
                {
                    $p_oController->get_oLayout()->add("oSession",$oSession);
                    return true;
                }
                
                //we have to evaluate if this controller and his actions are public or privated.
                //if is not allowed we have to show the validation page.
                $oNewView = $this->evaluate($oView->get_sParentController(), $oView->get_sView());
                if ($oNewView != null) {
                    //new window with the correspondent validation page
                    $p_oController->set_oView($oNewView);
                    //if we have to show a NewView we will have to change the controller and its action.
                    $someBeforeFilter = $p_oController->get_someBeforeFilter();
                    
                    $someBeforeFilter['oClass'] = $this;
                    $someBeforeFilter['sMethod'] = $oNewView->get_sView();

                    $p_oController->set_someBeforeFilter($someBeforeFilter);

                    return false;
                }
            }
            else { //the user is validated

                //Here, we must be validated. This new CApp, gets the previous instance of the session
                $oSession = new CAppSession();

                //now we have to check what kind of user is login and what controller and action is requesting
                //0. Root User Backend, 1. Normal User Frontend,  2.For example, Profesional User frontend
                $p_oController->get_oLayout()->add("oSession",$oSession);
                if ($oView->get_sView() != "close") {


                    if ($oSession->__get("typeadmin") == C_ADMIN_USER) {
                        return true;
                    }
                    
                    if ($oSession->__get("type") == C_USER_USER) {
                        //if you are a normal user you must access to user access
                        $bAllowed = $this->isZoneAllowed($oView->get_sParentController(),$oView->get_sView(),$oSession);
                        if (!$bAllowed) {
                              $oLayout = $p_oController->get_oLayout();
                            $oLayout->add("sTextMessage",__("Restricted Area for a User",$this->_sLanguage,true));
                            $oLayout->add("sTypeMessage",C_MESSAGE_ERROR);
                            redirect($this->link("unauthorized401","logins"),"","",true);
                            return false;
                        }
                    }
                    if ($oSession->__get("type") == C_PROF_USER) {
                        //if you are a professional user you must access as professional access
                        $bAllowed = $this->isZoneAllowed($oView->get_sParentController(),$oView->get_sView(),$oSession);
                        if (!$bAllowed) {
                            $oLayout = $p_oController->get_oLayout();
                            $oLayout->add("sTextMessage",__("Restricted Area for a Company",$this->_sLanguage,true));
                            $oLayout->add("sTypeMessage",C_MESSAGE_ERROR);
                            redirect($this->link("unauthorized401","logins"),"","",true);
                            return false;
                        }
                    }

                }
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return true;
    }

    public function unauthorized401() {
        
    }

    private function getTypeZone()
    {
        
    }
    
    
    /**
     * This function indicates if a user can access to a concrete zone. For example if a user
     * has logged in and wants to access to an admin zone, obviously he can't do it.
     * @param <type> $p_sController This parameter is the zone controller where the user wants to access
     * @param <type> $p_sAction This parameter is the controller action
     * @param <type> $p_oSession The session established by a user when a user log in.
     * @return <Boolean> This function returns true if a logged user can access to a zone or false otherwise
     */
    public function isZoneAllowed($p_sController, $p_sAction, $p_oSession){
        try {
           
           //First we check if the user has an active session or has logged in
           if ($p_oSession != null) {
               //we evaluate what kind of user is 0. Admin 1. User
               $iType = $p_oSession->__get("type");

               //if it's a public page or a page inside of logins, we don't have to continue, by default the zone is allowed.
               if (strcasecmp("page", $p_sController)==0) {
                   return true;
               }

               if (strcasecmp("logins", $p_sController)==0) {
                   return true;
               }

               
               if ($iType == C_USER_USER) { //For normal users
                   if (isset($this->_somePrivatedUserZones[$p_sController]) ) {
                       $sActions = $this->_somePrivatedUserZones[$p_sController];
                       //it's a user zone
                       //if the action controller is * or it's in the array then we show the login window
                       //if ((strcasecmp($this->_somePrivatedUserZones[$p_sController],$p_sAction) == 0) ||
                       if ((stristr($sActions,$p_sAction) != false) ||
                              (strcasecmp($this->_somePrivatedUserZones[$p_sController],"*")==0))
                       {
                            return true;
                       }
                       else {
                           return false;
                       }
                   }
                   else {
                       return false;
                   }
               }
               elseif ($iType == C_PROF_USER) { //For profesional users
                   if (isset($this->_somePrivatedProfZones[$p_sController]) ) {
                       $sActions = $this->_somePrivatedProfZones[$p_sController];
                       //it's a user zone
                       //if the action controller is * or it's in the array then we show the login window
                       //if ((strcasecmp($this->_somePrivatedProfZones[$p_sController],$p_sAction) == 0) ||
                       if ((stristr($sActions,$p_sAction) != false) ||
                              (strcasecmp($this->_somePrivatedProfZones[$p_sController],"*")==0))
                       {
                            return true;
                       }
                       else {
                           return false;
                       }
                   }
                   else {
                       return true;
                   }
               }
           }


        }
        catch(CException $e) {
            throw $e;
        }
        return false;
    }

    public function index() {

    }

    public function professional($p_sFrom = null) {
        if(!is_null($p_sFrom)){
            if($p_sFrom == C_LOGIN_TRIAL){
                $this->_oView->add('iLoginVersion',C_LOGIN_TRIAL);
            }
            else if( ( strpos(C_ROOT_DOMAIN, "demos") != false ) || C_DEMOS == 1){
                $this->_oView->add('iLoginVersion',C_LOGIN_DEMOS);
            }
            else{
                $this->_oView->add('iLoginVersion',C_LOGIN_APP);
            }
        }
        else if(C_DEMOS == 1){
            $this->_oView->add('iLoginVersion',C_LOGIN_DEMOS);
        }

        $oSession = new CAppSession();
        if (isset($oSession)) {
            if (($oSession->__get("type")!=false) && ($oSession->__get("login")!=false)) {
                redirect($this->link("index","dashboard"),"","",true);
                return true;
            }
        }
    }

    public function close($p_iFromAdmin = false) {
        try {
            $oSession = new CAppSession();            
            $iType = $oSession->__get("type");
            
            if ($p_iFromAdmin == true)
            {
                //this case is used when an admin click in sign out and 
                $iType = $oSession->__get("typeadmin");
            }
            
            $this->_oLayout->remove("oSession");            
            $oSession->delete("type");
            $oSession->clean();
            $this->unsetcookies();
            $sUrl = "logins/";
            switch($iType) {
                case C_ADMIN_USER:                  
                                    $oSession->delete("idadmin");
                                    $oSession->delete("loginadmin");
                                    $oSession->delete("typeadmin");                                    
                                    $oSession->delete("nameadmin");
                                    $oSession->delete("namecompanyadmin");
                                    $oSession->delete("someProfile");
                                    redirect($this->link("backend","logins"),"","",true);
                                    break;

                case C_USER_USER:   $sUrl = "logins/";
                                    redirect($sUrl);
                                    break;

                case C_PROF_USER:   
                default:            redirect($this->link("professional","logins"),"","",true);
                                    break;
            }            
        }
        catch(CException $e) {
            throw $e;
        }
        return true;
    }

    /**
     * This function solves the action /logins/loguser. It will be executed
     * when a normal user will try to log into your system.
     */
    public function loguser() {
        try {
      
        }
        catch(CException $e) {
            throw $e;
        }
        return false;
    }

    /**
     * This function solves the action /logins/logadmin. It will be executed
     * when an administrator user will try to log into our system
     */
    public function backend() {
        try {

            $oSession = new CAppSession();
            $someParameters = $this->get_someParameters();
            
            //if you are trying to validate
            if ((isset($someParameters['nameInpPassword'])) &&
                    (isset($someParameters['nameInpUser']))) {
                
                $sUser = $someParameters['nameInpUser'];
                $sPassword = md5($someParameters['nameInpPassword']);
                $iType = $someParameters["nameSelType"];
                //and if you don't have a session initiated we have to check it
                if ($oSession->__get("typeadmin")==false) {
                    
                    //it will ask to table admins
                    $oAdmin = $this->getUsedClass("CAdmin");
                    $somePartsAdmins = $oAdmin->getSomeAdmins($sUser, $sPassword, $iType);                                        
                    
                    if ($somePartsAdmins != null && count($somePartsAdmins) > 0) {
                        if (count($somePartsAdmins) > 0) {                             
                             //now it will get the profile for Admin                                                          
                             $oAdmProfile = $this->getUsedClass("CAdmprofile");
                             $someAdmProfile = $oAdmProfile->getProfileByIdUser($somePartsAdmins[0]['id']);
                             if (count($someAdmProfile)<=0)
                             {
                                $this->_oLayout->add("sTextMessage",__( "This user hasn't a profile",$this->_sLanguage,true));
                                $this->_oLayout->add("sTypeMessage",C_MESSAGE_ERROR);                                
                                return false;
                             }
                             else
                             {
                                 //if get_someData have data values, you will be in the database and we will create the session
                                 $oSession = new CAppSession();                             
                                 $oSession->__set("idadmin",$somePartsAdmins[0]['id']);
                                 $oSession->__set("loginadmin",$somePartsAdmins[0]['email']);                             
                                 $oSession->__set("typeadmin",C_ADMIN_USER);                              
                                 $oSession->__set("nameadmin",$somePartsAdmins[0]["contact_name"]);
                                 $oSession->__set("namecompanyadmin",$somePartsAdmins[0]["company_name"]);
                                 $oSession->__set("someProfile",$someAdmProfile);
                                 $oSession->__set("iLanguageId",7); //@todo: change
                                 $this->_oLayout->add("oSession",$oSession);
                                 if ($someAdmProfile[0]["id"] == C_PROFILE_CLIENT)
                                 {
                                    redirect($this->link("index","timelines"),__("You have been logged in correctly",$this->_sLanguage,true), C_MESSAGE_OK, true);
                                 }
                                 else
                                 {
                                    redirect($this->link("index","backusers"),__("You have been logged in correctly",$this->_sLanguage,true), C_MESSAGE_OK, true);
                                 }
                                 return true;
                             }

                        }
                        else {
                            //Second we have to set the view that will show the controller                            
                            $this->_oLayout->add("sTextMessage",__( "Login or password incorrect",$this->_sLanguage,true));
                            $this->_oLayout->add("sTypeMessage",C_MESSAGE_ERROR);
                            //$this->_oView->add("sMessage",__( "Login or password incorrect",$this->_sLanguage,true));
                            return false;
                        }
                    }
                }                
            }


            //if you arrive here and you have a previous session created
            if ($oSession->__get("typeadmin")!=false) {
                redirect($this->link("index","backusers"),__("You have been logged in correctly",$this->_sLanguage,true), C_MESSAGE_OK, true);
                return true;
            }

            //if you arrive here and don't have session
            $this->_oView->set_sView('backend.php');
            $this->_oView->add("sMessage",__( "Login or password incorrect",$this->_sLanguage,true)); 

        }
        catch(CException $e) {
            throw $e;
        }
        return false;
    }

    
    public function get_somePrivatedAdminZones() {
        try {
            return $this->_somePrivatedAdminZones;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function get_somePrivatedUserZones() {
        try {
            return $this->_somePrivatedUserZones;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function get_somePrivatedProfZones() {
        try {
            return $this->_somePrivatedProfZones;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function set_somePrivatedAdminZones($p_somePrivatedAdminZones) {
        try {
            $this->_somePrivatedAdminZones = $p_somePrivatedAdminZones;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function set_somePrivatedUserZones($p_somePrivatedUserZones) {
        try {
            $this->_somePrivatedUserZones = $p_somePrivatedUserZones;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function set_somePrivatedProfZones($p_somePrivatedProfZones) {
        try {
            $this->_somePrivatedProfZones = $p_somePrivatedProfZones;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    private function unsetcookies() {
        try {
            setcookie(C_COOKIE_USERID, "", time()+3600*24*365,"/","");
            setcookie(C_COOKIE_TYPEID, "", time()+3600*24*365,"/","");
            setcookie(C_COOKIE_SUBTYPEID, "", time()+3600*24*365,"/","");
        }
        catch(CException $e) {
            throw $e;
        }
    }

    /**
     * This function sets the cookies to remember the login of the user. This function is private, it will not be executed by the url.
     * @param <type> $p_iType the type of the user.
     * @return <type> if fails it return false, if success return true
     */
    private function setcookies($p_iType, $p_iId, $p_iSubtype) {
        try {
           $someParameters = $this->get_someParameters();
           //nameInpRemember
           //first it will check if there are parameters submitted and if is the parameter nameInpRemember
           if (isset($someParameters) && (isset($someParameters["nameInpRemember"])))
           {
                $oSession = new CAppSession();
                $iIdEncrypt = $oSession->encrypt($p_iId); //normally the user
                $iType = $oSession->encrypt($p_iType);
                $iSubtype = $oSession->encrypt($p_iSubtype);
                setcookie(C_COOKIE_USERID, $iIdEncrypt, time()+3600*24*365,"/",""); //the id of the user
                setcookie(C_COOKIE_TYPEID, $iType, time()+3600*24*365,"/",""); //type employee or user
                setcookie(C_COOKIE_SUBTYPEID, $iSubtype, time()+3600*24*365,"/",""); //the subtype of the employees, management, concierge, etc
                return true;
           }
        }
        catch(CException $e) {
            $this->unsetcookies(); //it's only used to avoid a loops
            throw $e;
        }
        return false;
    }

    private function checkcookies(&$p_oController) {
        try {
            //Compruebo si la cookie de usuario existe
            if(isset($_COOKIE[C_COOKIE_USERID]))
            {
                //Compruebo que no este vacia
                if($_COOKIE[C_COOKIE_USERID] != "")
                {
                    //Compruebo si existe el tipo de usuario
                    if(isset($_COOKIE[C_COOKIE_TYPEID]) && ($_COOKIE[C_COOKIE_TYPEID] != ""))
                    {
                        $oSession = new CAppSession();
                        $iId = $oSession->decrypt($_COOKIE[C_COOKIE_USERID]);
                        $iType = $oSession->decrypt($_COOKIE[C_COOKIE_TYPEID]);
                        $iSubType = $oSession->decrypt($_COOKIE[C_COOKIE_SUBTYPEID]);                        

                       //it will check the type of the user
                        if($iType == C_PROF_USER)
                        {                             
                             $oUser = $this->getUsedClass("CUser");
                             $oUser->get_someUsersById($iId);
                             $someData = $oUser->get_someData();
                             
                             if (count($someData)>0){
                                 $oSession->__set("id",$someData[0]['id']);
                                 $oSession->__set("login",$someData[0]['email']);
                                 $oSession->__set("type",C_PROF_USER); //0 it's the PROFESSIONAL user
                                 $oSession->__set("name",$someData[0]["name"]);
                                 $oSession->__set("logo",$someData[0]["logo"]);
                                 $oSession->__set("url",$someData[0]["url"]);
                                 $this->_oLayout->add("oSession",$oSession);
                                 $p_oController->get_oLayout()->add("iAvailableSMS",$someData[0]["cache_messages"]);                                
                                 return true;

                            }
                            else {
                                return false;
                            }
                        }
                        else //if is a user
                        {
                            if ($iType == C_USER) {                               
                                return false;
                            }
                        }
                    }
                }
            }
        }
        catch(CException $e) {
            $this->unsetcookies(); //it's only used to avoid a loops
            throw $e;
        }
        return false;

    }

}
?>
