<?php
/**
 * Class CController to be extended for all Controllers.
 * @author Nubesis http://www.nubesis.com
 * @copyright Nubesis S.L.
 */
class CController {

    /**
     * The content for wrapper template view.
     * @protected
     */
    protected $_oView;

    /**
     * The wrapper template view.
     * @protected
     */
    protected $_oLayout;

    /**
     * The type of view: C_FRONTEND, C_BACKEND, C_ELEMENTS, C_PAGES, C_ERRORS, C_API
     * @protected
     */
    protected $_sType = C_FRONTEND;
    
    /**
     * Parameters for controller.
     * @protected
     */
    protected $_someParameters = array();

    /**
     * List of parameters for GET Method.
     * @protected
     */
    protected $_sGetList = null;

    /**
     * The language selected.
     * English default
     * @protected
     */
    protected $_sLanguage = 'en';

    /**
     * Indicates if the language has passed by the url
     * true default = not passed
     * @protected
     */
    protected $_sLanguageDefault = true;

    /**
     * Plugins array.
     * @protected
     */
    protected $_somePlugins = array();

    /**
     * Plugins array.
     * @protected
     */
    protected $_someModels = array();

    /**
     * Plugins array.
     * @protected
     */
    protected $_someControllers = array();

    /**
     * Layout to use, layout.php by default.
     * @protected
     */

    protected $_sLayout = null;

    /**
     * When some controller want to use the same view for all methods
     * @protected
     */
    protected $_sFixedView = null;

    /**
     * When some controller want to use the same method for all views
     * @protected
     */
    protected $_sFixedMethod = null;

    /**
     * When some controller want to use his own error method for not found methods
     * @protected
     */
    protected $_sErrorMethod = null;


    protected $_someBeforeFilter = array();

    protected $_someErrorMessages = array();
    protected $_iTypeMessage = -1;

    protected $someCss = array();
    protected $someJavascripts = array();

    /**
     * Constructor. Create the wrapper template.
     */
    public function CController() {
        $this->_someCoreHelpers = array( "CHtml" , "CPaginator" , "CAjax" );
        $this->_oLayout = new CAppView( "layouts/layout.php" );
        $this->getModels();
        $this->getControllers();
    }

    /**
     * @assert (null) == false
     * @assert ('texto') == false
     * @assert ('') == false
     *
     * Display the content wrapped in the main template. Subclasses should
     * invoke this method when they want to present their content to the screen.
     *
     * @param $p_oView the content template to display
     */
    public function display($p_oContent) {
        try
        {
            $this->getUsedHelpers();
            if($p_oContent != null && !is_string($p_oContent))
            {
                $p_oContent->add('sLang',$this->_sLanguage);
                
                if (count($this->get_someErrorMessages())>0) {
                    $this->_oLayout->add('sTextMessage',$this->get_someErrorMessages());
                    $this->_oLayout->add('sTypeMessage',$this->get_iTypeMessage());
                    $this->_oLayout->add('someErrorMessages',$this->get_someErrorMessages());
                }
                $this->_oLayout->add('content', $p_oContent);
                $this->_oLayout->add("sLang",$this->_sLanguage);
                echo $this->_oLayout->fetch();
            }
            else
            {
                return false;
            }
        }
        catch(CException $e) {
            throw $e;
        }
        return true;
    }
    /**
     * Invoked when a GET request is routed to the controller.
     */
    public function renderVariablesGet()
    {
        try
        {
            $bReturnValue = false;
            if ($this->setSomeParameters($_GET, 'GET') == true)
            {
                $this->callControllerMethod();
                $bReturnValue = true;
            }
        }
        catch(CException $e) {
            throw $e;
        }
        return $bReturnValue;
    }
    /**
     * Invoked when a POST request is routed to the controller.
     */
    public function renderVariablesPost()
    {
        try
        {
            $bReturnValue = false;
            if ($this->setSomeParameters($_POST, 'POST') == true)
            {
                $this->callControllerMethod();
                $bReturnValue = true;
            }
        }
        catch(CException $e) {
            throw $e;
        }
        return $bReturnValue;
    }
   
    /**
     * Verify that a parameter is of the correct type.
     *
     * @param $p_sTtype the type of the parameter, either 'mixed', 'int' or 'string'
     * @param $p_sValue the value of the parameter
     * @return true or false.
     */
    protected function _validateParam($p_sType, $p_sValue) {
        try
        {
            if ($p_sType == 'mixed') {
                return true;
            }

            return $p_sType == 'int' && is_numeric($p_sValue)
                    || $p_sType == 'string' && is_string($p_sValue);
        }
        catch(CException $e) {
            throw $e;
        }
    }
    /**
     * Public method that displays a 404 Not Found page.
     */
    protected function _showError() {
        @header("HTTP/1.0 404 Not Found");
        include(C_FILE_NOT_FOUND);
        exit;
    }

    /**
     * Render GET params when $_sGetList is defined.
     *
     * @param array $p_someParameters parameters passed for controller
     * @param string $p_sGetList string with de variables name to parse GET paramas
     */
    protected function _renderGetListParameters($p_someParameters, $p_sGetList)
    {
        try
        {
            $someKeys = explode(',', $p_sGetList);

            for ($i = 0, $l = count($p_someParameters); $i < $l; $i++) {
                if (!empty($p_someParameters[$i])) {
                    if(isset ($someKeys[$i]))
                    {
                        list($sType, $sName) = explode(' ', trim($someKeys[$i]));
                        $sValue = urldecode(trim($p_someParameters[$i]));

                        if (!$this->_validateParam(trim($sType), $sValue)) {
                            $this->_showError();
                        }
                        $this->_someParameters[trim($sName)] = $sValue;
                    }
                    else
                    {
                        $sValue = urldecode(trim($p_someParameters[$i]));
                        $this->_someParameters[trim($i)] = $sValue;
                    }
                }
            }
        }
        catch (CException $e) {
            throw $e;
        }
    }
    /**
     * Render GET or POST params when $_sGetList is not defined.
     *
     * @param array $p_someParameters parameters passed for controller
     */
    protected function _renderNormalListParameters($p_someParameters)
    {
        try
        {
            foreach($p_someParameters as $sIndex=>$sParameter) {
                $this->_someParameters[$sIndex] = $sParameter;
            }
        }
        catch(CException $e) {
            throw $e;
        }
    }

    protected function callControllerMethod()
    {
        try
        {
            $sClass = get_class($this);

            $someClassMethods = get_class_methods($sClass);

            $sMethod = 'index';

            if(isset ( $this->_sDefaultMethod ) )
            {
                $sMethod = $this->_sDefaultMethod;
            }

            if( isset ($this->_someParameters[0]) && !empty($this->_someParameters[0]) )
            {
                $sMethod = $this->_someParameters[0];
            }

            if( $this->_sFixedMethod != null)
            {
                $sMethod = $this->_sFixedMethod;
            }

            if(in_array($sMethod,$someClassMethods))
            {
                //Select and add actual_page to layout
                if($this->_sFixedView != null)
                {
                    $page = $this->_sFixedView;
                    $this->_oLayout->add('page',$page);
                }
                else
                {
                    $page = $sMethod.'.php';
                    $this->_oLayout->add('page',$page);
                }

                $sControllerView = strtolower(substr(str_replace(C_CONTROLLER_CLASS, '', $sClass), 1))."/";
                $sUrlController = strtolower(substr(str_replace(C_CONTROLLER_CLASS, '', $sClass), 1));

                //Get and add content page
                $sViewActionPath = C_VIEWS . $this->_sType.$sControllerView;

                if($this->_sType == C_PLUGINS)
                {
                    $sViewActionPath = C_PLUGINS . $sControllerView ."views/";
                }

                $this->_oView = new CAppView( $page , $sViewActionPath);

                //Set title page
                $this->setTitle(C_TITLE_PAGE);
                $this->setMetaKeywords(C_METAKEYWORDS);
                $this->setMetaDescription(C_METADESCRIPTION);

                //Delete first parameter from array because is the action
                if( $this->_sFixedMethod == null)
                {
                    //If it has $_sFixedMethod it means that the action does not come by the url
                    array_splice($this->_someParameters, 0, 1);
                }

                $haveMethodPlugin = false;

                //Check if controller has some actions reimplemented in some plugin
                foreach ( $this->_somePlugins as $sPluginIndex => $oPlugin )
                {
                    if(array_key_exists($sMethod,$oPlugin))
                    {
                        $haveMethodPlugin = true;

                        //Get the plugin name controller
                        $sNamePluginController = C_CLASS_FILE . ucfirst ( $sPluginIndex ) . C_CONTROLLER_CLASS . ".php";
                        $sNamePluginClass = C_CLASS_FILE . ucfirst ( $sPluginIndex ) . C_CONTROLLER_CLASS;

                        //Require the controller and create an instance
                        require_once(C_PLUGINS . $sPluginIndex . "/controllers/" . $sNamePluginController);
                        $oClass = basename($sNamePluginClass);
                        $oPluginController = new $oClass();

                        $sViewActionPath = C_PLUGINS . $sPluginIndex ."/views/";

                        //Assign to the controller the method/view to show
                        $oPluginController->_oView = new CAppView( $oPlugin[$sMethod] . ".php" , $sViewActionPath);
                        //$oPluginController->_oView->set_sParentUrl($sUrlController, $this->_sLanguage, $this->_sLanguageDefault);
                        $this->set_sParentUrl($oPluginController, $sUrlController);
                        
                        //Set title page
                        $oPluginController->setTitle(C_TITLE_PAGE);
                        $oPluginController->setMetaKeywords(C_METAKEYWORDS);
                        $oPluginController->setMetaDescription(C_METADESCRIPTION);

                        //Call method
                        $this->callAction($oPluginController, $oPlugin[$sMethod], $this->_someParameters);

                        if( $oPluginController->_sLayout != null )
                        {
                            $oPluginController->_oLayout = new CAppView('layouts/'. $oPluginController->_sLayout .'.php');
                        }

                        //Display view
                        $oPluginController->display($oPluginController->_oView);
                    }
                }

                if(!$haveMethodPlugin)
                {
                    //$this->_oView->set_sParentUrl($sUrlController, $this->_sLanguage, $this->_sLanguageDefault);
                    $this->set_sParentUrl($this, $sUrlController);

                    //Call Method
                    $this->callAction($this, $sMethod, $this->_someParameters);

                    if( $this->_sLayout != null )
                    {
                        $this->_oLayout->set_sView('layouts/'. $this->_sLayout .'.php');
                    }

                    $this->display($this->_oView);
                }
            }
            else
            {
                if($this->_sErrorMethod != null)
                {
                    //$this->_oView->set_sParentUrl('errors', $this->_sLanguage, $this->_sLanguageDefault);
                    $this->set_sParentUrl($this, "errors");
                    //Call Error Method
                    $this->callAction($this, $this->_sErrorMethod, $this->_someParameters);

                    if( $this->_sLayout != null )
                    {
                        $this->_oLayout->set_sView('layouts/'. $this->_sLayout .'.php');
                    }
                    $this->display($this->_oView);
                }
                else
                {
                    //Select and add actual_page to layout
                    $page = '404.php';

                    if( $this->_sLayout != null )
                    {
                        $this->_oLayout->set_sView('layouts/'. $this->_sLayout .'.php');
                    }

                    $this->_oLayout->add('page',$page);

                    //Get and add content page
                    $this->_oView = new CAppView($page,C_VIEWS . 'errors/error/');

                    $this->_oView->add('sErrorMessage',"The action doesn't exists.");

                    //Set title page
                    $this->setTitle(C_TITLE_PAGE);
                    $this->setMetaKeywords(C_METAKEYWORDS);
                    $this->setMetaDescription(C_METADESCRIPTION);

                    //$this->_oView->set_sParentUrl('errors', $this->_sLanguage, $this->_sLanguageDefault);
                    $this->set_sParentUrl($this, "errors");
                    $this->display($this->_oView);
                }
            }

            return true;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    protected function callAction($p_oClass, $p_sMethod, $p_someParameters)
    {
        try {
            $this->setActiveMethod( $p_sMethod );
            $this->_someBeforeFilter = array("oClass" => $p_oClass , "sMethod" => $p_sMethod, "someParameters" => $p_someParameters);

            //Call beforeFilter
            call_user_func_array(array($p_oClass, 'beforeFilter'), $p_someParameters);

            //Call to the reimplemented method
            call_user_func_array(array($this->_someBeforeFilter['oClass'], $this->_someBeforeFilter['sMethod']),
                    $this->_someBeforeFilter['someParameters']);

            $oSession = new CAppSession();

            if ( $oSession->__get("message") )
            {
                $this->_oLayout->add('sTextMessage',__( $oSession->__get("message") , $this->_sLanguage , true ) );
                $this->_oLayout->add('sTypeMessage',$oSession->__get("messageType"));
                $oSession->delete("message");
                $oSession->delete("messageType");
            }

            if ( $oSession->__get("sLastInserted") && $p_sMethod == "index")
            {
                $this->_oView->add('sLastInserted' , $oSession->__get("sLastInserted") );
                $oSession->delete("sLastInserted");
            }
            
            if( isset( $this->_someParameters['data'] ) && !$this->_oView->issetSomeData() )
                $this->_oView->add('someData',$this->_someParameters['data']);
            
            $this->_oLayout->add('someCss',$this->someCss);
            $this->_oLayout->add('someJavascripts',$this->someJavascripts);
            
            call_user_func_array(array($this->_someBeforeFilter['oClass'], 'afterFilter'),
                    $this->_someBeforeFilter['someParameters']);
        }
        catch(CException $e) {
            throw $e;
        }
        
    }

    protected function paginate($p_sModel , $p_sMethod , $p_sController , $p_someSelectValues)
    {
        try{
            //$this->_oView->set_sParentUrl($p_sController, $this->_sLanguage,$this->_sLanguageDefault);
            $this->set_sParentUrl($this, $p_sController);
            
            $oObject = $this->getUsedClass($p_sModel);

            $p_someSelectValues["p_bPagination"] = true;

            if (isset($p_someSelectValues["p_sColumns"]) && ($p_someSelectValues["p_sColumns"] != ""))
            {
                $p_someSelectValues["p_sColumns"] = "SQL_CALC_FOUND_ROWS ".$p_someSelectValues["p_sColumns"];
            }
            else
            {
                $p_someSelectValues["p_sColumns"] = "SQL_CALC_FOUND_ROWS *";
            }


            $oObject->select($p_someSelectValues);
            $someValues = $oObject->get_someData();
            //now it gets the total of rows used in the query without pagination to calculate the pagination
            $iTotalRows = $oObject->getTotalRows();
            $iMaxPage = $iTotalRows;

            if(isset($p_someSelectValues["p_sLimit"]))
                $p_sLimit = $p_someSelectValues["p_sLimit"];
            else
                $p_sLimit = null;

            if($p_sLimit != null)
            {
                if ($iMaxPage >= 1)
                {
                    if($iMaxPage%$p_sLimit > 0)
                        $iMaxPage = ($iMaxPage - $iMaxPage%$p_sLimit )/$p_sLimit + 1;
                else
                        $iMaxPage = $iMaxPage/$p_sLimit;
                }
                else
                {
                    $iMaxPage = 0;
                }
            }
            else
            {
                if ($iMaxPage >= 1)
                {
                    if($iMaxPage%10 > 0)
                        $iMaxPage = floor($iMaxPage/10 + 1);
                    else
                        $iMaxPage = floor($iMaxPage/10);
                }
                else
                {
                    $iMaxPage = 0;
                }
            }

            $this->_oView->add('iMaxPage',$iMaxPage);
            if (isset($iTotalRows[0]['count']))
            {
                $this->_oView->add('iTotalRows', $iTotalRows[0]['count']);
            }
            else
            {
                $this->_oView->add('iTotalRows', 0);
            }
            $this->_oView->add('iLimit', $p_someSelectValues["p_sLimit"]);

           
        }
        catch(CException $e) {
            throw $e;
        }
         return $someValues;
    }

    protected function beforeFilter()
    {
        //Implement in controller
    }

    protected function afterFilter()
    {
        //Implement in controller
    }


    /**
     * This function forces CDispatcher to do a redirection to a new Controller.
     * @param <String> $p_sNameControllerDestiny The name of the class (CClass) controller where we want go
     * @param <String> $p_sNameParentController The name of the controller in the url where we are going to go
     * @param <String> $p_sView The view destiny where we want to go
     * @param <String> $p_sPathView The view path url where we want to go
     * @return <Boolean> It returns true if all went ok
     */
    protected function redirect($p_sNameControllerDestiny, $p_sNameParentController, $p_sPathView, $p_sView="index") {
        try {
            $sUrl = C_ROOT_KEY . $p_sNameParentController . "/" .$p_sView;
            redirect( $sUrl );

//                $oNewController = $this->getUsedClass($p_sNameControllerDestiny);
//
//                $oNewView = new CAppView($p_sView.".php",$p_sPathView);
//                $oNewView->set_sParentController($p_sNameParentController);
//
//                $this->set_oView($oNewView);
//                $oNewController->set_oView($oNewView);
//
//                $this->_someBeforeFilter['oClass'] = $oNewController;
//                $this->_someBeforeFilter['sMethod'] = "$p_sView";
        
        }
        catch(CException $e) {
            throw $e;
        }
        return $oNewController;
    }

    /**
     * GETTERS
     */

    /**
     * Create default model object for this controller
     */
    public function getModels() {
        try
        {
            $sControllerName = get_class($this);
            $sModelName = "";
            if(preg_match( "/sController/" , $sControllerName))
            {
                $sModelName = str_replace("sController", "", $sControllerName);
            }
            else
            {
                $sModelName = str_replace("Controller", "", $sControllerName);
            }
            $sObjectModelVar = "_o" . substr( $sModelName, 1 );

            $oModel = $this->getUsedClass($sModelName);

            if( $oModel )
            {
                $this->{$sObjectModelVar} = $this->getUsedClass($sModelName);

                $someMethods = get_class_methods($sModelName);

                if($someMethods != null)
                {
                    if(in_array( "setController" , $someMethods ) )
                    {
                        $this->{$sObjectModelVar}->setController($this);
                    }
                }
            }
        }
        catch(CException $e) {
            throw $e;
        }
    }

    /**
     * Create used models objects if controller indicate it
     */
    public function getUsedModels() {
        try
        {
            foreach ($this->_someModels as $sModel)
            {
                $sObjModelVar = "_o" . substr( $sModel, 1 );
//                $sObjModelVar = str_replace("C", "_o", $sModel);

                $oModel = $this->getUsedClass($sModel);

                if( $oModel )
                {
                    $this->{$sObjModelVar} = $oModel;

                    $someMethods = get_class_methods($sModel);

                    if($someMethods != null)
                    {
                        if(in_array( "setController" , $someMethods ) )
                        {
                            $this->{$sObjModelVar}->setController($this);
                        }
                    }
                }

//                $this->{$sObjModelVar} = $this->getUsedClass($sModel);
//                $someMethods = get_class_methods($sModel);
//
//                if($someMethods != null)
//                {
//                    if(in_array( "setController" , $someMethods ) )
//                    {
//                        $this->{$sObjectModelVar}->setController($this);
//                    }
//                }
            }
        }
        catch(CException $e) {
            throw $e;
        }
    }

    /**
     * Create used controllers objects if controller indicate it
     */
    public function getControllers() {
        try
        {
            foreach ($this->_someControllers as $sController)
            {
                $sObjControllerVar = "_o" . substr( $sController, 1 );

                $oController = $this->getUsedClass($sController);

                if( $oController )
                {
                    $sControllerView = substr( $sController, 1 );
                    $sControllerView = substr($sControllerView, 0 , ( strlen( $sControllerView ) - 10 ) );

                    //Get and add content page
                    $sViewActionPath = C_VIEWS . $oController->_sType.$sControllerView;

                    if($oController->_sType == C_PLUGINS)
                    {
                        $sViewActionPath = C_PLUGINS . $sControllerView ."views/";
                    }

                    //Select and add actual_page to layout
                    if($oController->_sFixedView != null)
                        $page = $oController->_sFixedView;
                    else
                        $page = 'index.php';

                    $oController->_oView = new CAppView( $page , $sViewActionPath);

                    $this->{$sObjControllerVar} = $oController;

                    $someMethods = get_class_methods($sController);

                    if($someMethods != null)
                    {
                        if(in_array( "getUsedModels" , $someMethods ) )
                        {
                            $this->{$sObjControllerVar}->getUsedModels();
                        }

                        if(in_array( "setLanguage" , $someMethods ) )
                        {
                            $this->{$sObjControllerVar}->setLanguage($this->_sLanguage, $this->_sLanguageDefault);
                        }
                    }
                }
            }
        }
        catch(CException $e) {
            throw $e;
        }
    }

    /**
     * Set Active Method to Used Models
     */
    public function setActiveMethod( $p_sMethod ) {
        try
        {
            $this->setActiveMethodDefault( $p_sMethod );
            foreach ($this->_someModels as $sModel)
            {
                $sObjModelVar = "_o" . substr( $sModel, 1 );
                $this->{$sObjModelVar}->setActiveMethod( $p_sMethod );
            }
        }
        catch(CException $e) {
            throw $e;
        }
    }

    /**
     * Set Active Method to Default Model
     */
    public function setActiveMethodDefault( $p_sMethod ) {
        try
        {
            $sControllerName = get_class($this);
            $sModelName = "";
            if(preg_match( "/sController/" , $sControllerName))
            {
                $sModelName = str_replace("sController", "", $sControllerName);
            }
            else
            {
                $sModelName = str_replace("Controller", "", $sControllerName);
            }
            $sObjectModelVar = "_o" . substr( $sModelName, 1 );
            if( isset ( $this->{$sObjectModelVar} ) )
            {
                $this->{$sObjectModelVar}->setActiveMethod( $p_sMethod );
            }
        }
        catch(CException $e) {
            throw $e;
        }
    }

    /**
     * Add Helpers to View
     */
    public function getUsedHelpers() {
        try
        {
            $someCoreHelpers = array();

            if( isset ( $this->_someCoreHelpers ) )
                $someCoreHelpers = $this->_someCoreHelpers;

            $someHelpers = array();

            if( isset ( $this->_someHelpers ) )
                $someHelpers = $this->_someHelpers;

            $someCombinedHelpers = array_merge( $someCoreHelpers , $someHelpers );

            $someGetHelpers = array();

            foreach ( $someCombinedHelpers as $sHelper )
            {
                $sObjHelper = str_replace("C", "o", $sHelper);
                $oHelper = $this->getUsedClass( $sHelper . "Helper" );
                $someGetHelpers[] = $sObjHelper;
                $this->_oLayout->add( $sObjHelper , $oHelper );
                $this->_oView->add( $sObjHelper , $oHelper );
            }

            foreach ( $someGetHelpers as $sHelper )
            {
                $oHelper = $this->_oLayout->get( $sHelper );
                $oHelper->set_oLayout( $this->_oLayout );
                $this->_oLayout->set( $sHelper , $oHelper );

                $oHelper = $this->_oView->get( $sHelper );
                $oHelper->set_oView( $this->_oView );
                $this->_oView->set( $sHelper , $oHelper );
            }
        }
        catch(CException $e) {
            throw $e;
        }
    }

    /**
     * A simply list with parameters that will be used for controller
     * @return array with parameters for controller
     */
    public function get_someParameters()
    {
        return $this->_someParameters;
    }

     /**
     * Function to get type view. FRONTEND, BACKEND...
     */
    public function getType()
    {
        try {
            return $this->_sType;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function get_someBeforeFilter() {
        try
        {
            return $this->_someBeforeFilter;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function get_oView() {
        try {
            return $this->_oView;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function get_oLayout() {
        try
        {
            return $this->_oLayout;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    /**
     * Render GET or POST params when $_sGetList is not defined.
     *
     * @param array $p_someParameters parameters passed for controller
     */
    protected function getComponent($p_sComponent)
    {
        try
        {
            require_once($p_sComponent . ".php");
            $oClass = basename($p_sComponent);
            $oComponent = new $oClass($this);
            return $oComponent;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function getUsedClass($p_sClass)
    {
        try
        {

            //First, we have to find if p_sClass is included or we have to include it
            $sFilename = $p_sClass.".php";
            $sPaths = explode(PATH_SEPARATOR, get_include_path());
            $bFound = false;
            foreach ($sPaths as $sPath) {
                if (substr($sPath, -1) == DIRECTORY_SEPARATOR) {
                    $sFullPath = $sPath.$sFilename;
                } else {
                    $sFullPath = $sPath.DIRECTORY_SEPARATOR.$sFilename;
                }
                if (file_exists($sFullPath)) {
                    $bFound = true;
                    break;
                }
            }

            if($bFound)
            {
                require_once($p_sClass . ".php");
            }
            else {
                $oDir = opendir(C_PLUGINS);

                $someControllers = array();

                while ($sElement = readdir($oDir))
                {
                    if(!preg_match('/\./',$sElement))
                    {
                        if(file_exists(C_PLUGINS . $sElement."/models/".$p_sClass . ".php"))
                        {
                            require_once($sElement."/models/".$p_sClass . ".php");
                            $bFound = true;
                            break;
                        }

                        if(file_exists(C_PLUGINS . $sElement."/controllers/".$p_sClass . ".php"))
                        {
                            require_once($sElement."/controllers/".$p_sClass . ".php");
                            $bFound = true;
                            break;
                        }
                    }
                }

                closedir($oDir);
            }

            if( $bFound )
            {
                $oClass = basename($p_sClass);
                $oUsedClass = new $oClass();
                return $oUsedClass;
            }
            else
            {
                return false;
            }
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function get_someErrorMessages() {
        try {
            return $this->_someErrorMessages;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function get_iTypeMessage() {
        try {
            return $this->_iTypeMessage;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function get_sLanguage() {
        try {
            return $this->_sLanguage;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    /**
     * SETTER
     */

    /**
     * Set the page title.
     * @param $title the page title
     */
    public function setTitle($p_sTitle) {
        try
        {
            $this->_oLayout->add('sPageTitle', $p_sTitle);
        }
        catch(CException $e) {
            throw $e;
        }
    }

    /**
     * Set the page title.
     * @param $title the page title
     */
    public function setMetaKeywords($p_sMetaKeywords) {
        try
        {
            $this->_oLayout->add('sPageMetaKeywords', $p_sMetaKeywords);
        }
        catch(CException $e) {
            throw $e;
        }
    }

    /**
     * Set the page title.
     * @param $title the page title
     */
    public function setMetaDescription($p_sMetaDescription) {
        try
        {
            $this->_oLayout->add('sPageMetaDescription', $p_sMetaDescription);
        }
        catch(CException $e) {
            throw $e;
        }
    }

    /**
     * Set the language.
     * @param $title the page title
     */
    public function setLanguage($p_sLanguage, $p_sLanguageDefault) {
        try
        {
          //  $this->_oLayout->set_sLanguage($p_sLanguage);
          //  $this->_oLayout->set_sLanguageDefault($p_sLanguageDefault);
            $this->_sLanguage = $p_sLanguage;
            $this->_sLanguageDefault = $p_sLanguageDefault;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    /**
     * This function sets $_sGetList attribute of CController class. This parameter
     * must content a list of parameters with comma separations like int number_a, string param_b, etc
     * @param <type> $p_sGetList String with comma separated values. int param_a, int param_b
     */
    public function set_sGetList($p_sGetList) {
        try
        {
            $this->_sGetList = $p_sGetList;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    /**
     * @assert (array('param1' => 'value1','param2' => 'value2'),'GET') == true
     * @assert (array('param1' => 'value1','param2' => 'value2'),'POST') == true
     * @assert (array('param1' => 'value1','param2' => 'value2'),null) == true
     * @assert ('texto','GET') == true
     *
     * This function sets in a Hash Table the parameters passed in a form
     * @param array $p_someParameters parameters passed for controller
     * @param string $p_sMethod call Method: GET or POST
     * @return HashTable with pairs Index/Value
     */
    public function setSomeParameters($p_someParameters, $p_sMethod)
    {
        try
        {
            $bReturnValue = false;

            if(is_array($p_someParameters))
            {
                if(strcasecmp($p_sMethod,'GET')==0)
                {
                    $this->_renderNormalListParameters($p_someParameters);
                }
                else
                {
                    $this->_renderNormalListParameters($_GET);
                    $this->_renderNormalListParameters($p_someParameters);
                }
            }
            $bReturnValue = true;
        }
        catch(CException $e) {
            throw $e;
        }
        return $bReturnValue;
    }

     /**
     * Function to set type view. FRONTEND, BACKEND...
     */
    public function setType($p_sType)
    {
        try {
            $this->_sType = $p_sType;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function set_oView($p_oNewView) {
        try {
            $this->_oView = $p_oNewView;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function set_oLayout($p_oLayout) {
        try {
            $this->_oLayout = $p_oLayout;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function set_someBeforeFilter($p_someBeforeFilter) {
        try
        {
            $this->_someBeforeFilter = $p_someBeforeFilter;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    /**
     * This function is used to change the controller and refresh the value for the new selected language
     * @param <type> $p_oController new object with the new Controller.
     * @param <type> $p_sController The name, in string format, of the new CController object.
     *
     */
    public function set_sParentUrl($p_oController, $p_sController) {
        try {
            $p_oController->_oView->set_sParentUrl($p_sController, $this->_sLanguage,$this->_sLanguageDefault);
            $p_oController->_oLayout->set_sParentUrl($p_sController, $this->_sLanguage,$this->_sLanguageDefault);
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function set_someErrorMessages($p_someErrorMessages) {
        try {
            $this->_someErrorMessages = $p_someErrorMessages;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function set_iTypeMessage($p_iTypeMessage) {
        try {
            $this->_iTypeMessage = $p_iTypeMessage;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    /**
    * This function creates a link in the an href tag.
    * @param <type> $p_sView It's the view where will which will be used in the link
    * @param <type> $p_sController It's the controller which will be used in the link
    * @param <type> $p_sLanguageCustom It's the language which will be created the link
    * @return <type> it returns a url text
    */
   public function link($p_sView = '' , $p_sController = null, $p_sLanguageCustom = "", $p_bAbsoluteUrl = false)
   {
       try {

            if( defined ( 'C_ENCRYPT' ) && C_ENCRYPT == 1 )
            {
                $oSession = new CAppSession();
                if (($p_sController == "widgets") || ($p_sController == "widgetsjs") || ($p_sController == "csvs"))
                {
                    if ($p_bAbsoluteUrl == false) {
                        if ($p_sLanguageCustom == "") {
                            return C_ROOT_KEY.$this->_oView->showParentUrl($p_sController).$p_sView;
                        }
                        else {
                            return C_ROOT_KEY.$this->_oView->showParentUrlLanguage($p_sLanguageCustom, $p_sController).$p_sView;
                        }
                    }
                    else
                    {
                        if ($p_sLanguageCustom == "") {
                            return C_WIDGET_PROTOCOL.'://'.C_ROOT_DOMAIN.C_ROOT_KEY.$this->_oView->showParentUrl($p_sController).$p_sView;
                        }
                        else {
                            return C_WIDGET_PROTOCOL.'://'.C_ROOT_DOMAIN.C_ROOT_KEY.$this->_oView->showParentUrlLanguage($p_sLanguageCustom, $p_sController).$p_sView;
                        }
                    }
                }


                if($p_sController == "pages")
                {
                    if ($p_bAbsoluteUrl == false) {
                        if ($p_sLanguageCustom == "") {
                            return C_ROOT_KEY.$oSession->encrypt( $p_sView );
                        }
                        else {
                            return C_ROOT_KEY.$oSession->encrypt( $p_sView );
                        }
                    }
                    else
                    {
                        if ($p_sLanguageCustom == "") {
                            return C_WIDGET_PROTOCOL.'://'.C_ROOT_DOMAIN.C_ROOT_KEY.$oSession->encrypt( $p_sView );
                        }
                        else {
                            return C_WIDGET_PROTOCOL.'://'.C_ROOT_DOMAIN.C_ROOT_KEY.$oSession->encrypt( $p_sView );
                        }
                    }
                }
                else
                {
                    if ($p_bAbsoluteUrl == false) {
                        if ($p_sLanguageCustom == "") {
                            return C_ROOT_KEY.$oSession->encrypt($this->_oView->showParentUrl($p_sController).$p_sView);
                        }
                        else {
                            return C_ROOT_KEY.$oSession->encrypt($this->_oView->showParentUrlLanguage($p_sLanguageCustom, $p_sController).$p_sView);
                        }
                    }
                    else
                    {
                        if ($p_sLanguageCustom == "") {
                            return C_WIDGET_PROTOCOL.'://'.C_ROOT_DOMAIN.C_ROOT_KEY.$oSession->encrypt($this->_oView->showParentUrl($p_sController).$p_sView);
                        }
                        else {
                            return C_WIDGET_PROTOCOL.'://'.C_ROOT_DOMAIN.C_ROOT_KEY.$oSession->encrypt($this->_oView->showParentUrlLanguage($p_sLanguageCustom, $p_sController).$p_sView);
                        }
                    }
                }
            }
            else
            {
                if($p_sController == "pages")
                {
                    if ($p_bAbsoluteUrl == false) {
                        if ($p_sLanguageCustom == "") {
                            return C_ROOT_KEY.$p_sView;
                        }
                        else {
                            return C_ROOT_KEY.$p_sView;
                        }
                    }
                    else
                    {
                        if ($p_sLanguageCustom == "") {
                            return C_WIDGET_PROTOCOL.'://'.C_ROOT_DOMAIN.C_ROOT_KEY.$p_sView;
                        }
                        else {
                            return C_WIDGET_PROTOCOL.'://'.C_ROOT_DOMAIN.C_ROOT_KEY.$p_sView;
                        }
                    }
                }
                else
                {
                    if ($p_bAbsoluteUrl == false) {
                        if ($p_sLanguageCustom == "") {
                            return C_ROOT_KEY.$this->_oView->showParentUrl($p_sController).$p_sView;
                        }
                        else {
                            return C_ROOT_KEY.$this->_oView->showParentUrlLanguage($p_sLanguageCustom, $p_sController).$p_sView;
                        }
                    }
                    else
                    {
                        if ($p_sLanguageCustom == "") {
                            return C_WIDGET_PROTOCOL.'://'.C_ROOT_DOMAIN.C_ROOT_KEY.$this->_oView->showParentUrl($p_sController).$p_sView;
                        }
                        else {
                            return C_WIDGET_PROTOCOL.'://'.C_ROOT_DOMAIN.C_ROOT_KEY.$this->_oView->showParentUrlLanguage($p_sLanguageCustom, $p_sController).$p_sView;
                        }
                    }
                }
            }
       }
       catch(CException $e) {
           throw $e;
       }
   }

   public function showMessage( $p_CType , $p_sText = "" , $p_sView = null )
   {
        if(!is_null($p_sView))
            $this->_oView->set_sView('index.php');
        $this->_oLayout->add('sTextMessage',$p_sText);
        $this->_oLayout->add('sTypeMessage',$p_CType);
   }

}
?>
