<?php
/**
 * CDispatcher allows to map a resquest to the Controllers.
 * It will be the responsable to attend a call and yield the control to a function in controllers
 * by REQUEST METHOD
 * @author Nubesis http://www.nubesis.com
 * @copyright Nubesis S.L.
 */

class CDispatcher { 

    /**
     * The application configuration.
     * @access private
     */
    private $_someControllers;

    /**
     * The URL that will be dispatched.
     * @access private
     */
    private $_sUrl;

    /**
     * The language selected.
     * English default
     * @access private
     */
    private $_sLanguage = 'en';

    /**
     * Indicates if the language has passed by the url
     * true default = not passed
     * @access private
     */
    private $_sLanguageDefault = true;

    /**
     * It creates a new dispatcher.
     *
     * @param $p_sControlFile the location of the .ini file with the routes of the controllers
     */
    public function CDispatcher($p_sControlFile = null) {
        try {
            $oConfiguration = new CAppIni();
            $this->_someControllers = $oConfiguration->get_someInitValue();
        }
        catch(CException $e) {
            $this->manageLog($e);
        }
    }

    /**
     * @assert (C_ROOT_KEY."saasdfpi/") == true
     * @assert ("asdasdfasdf") == false
     * @assert ("") == false
     * @assert (null) == false
     * Dispatch the requested URL like a parameters. For each / we indicate a parameter.
     * @param $p_sUrl the URL to dispatch
     */
    public function dispatch($p_sUrl) {
        try
        {

            $p_sUrl = $this->encryptUrl($p_sUrl);

            $p_sUrl = $this->_detectGetVars($p_sUrl);

            $p_sUrl = $this->selectLanguage($p_sUrl);

            $oSession = new CAppSession();

            if($oSession->__get("userproflanguage")){
                $sLanguage = $oSession->__get("userproflanguage");
                $this->_sLanguage = $sLanguage;
            }

            //now it will load the language in the dictionary
            $sLanguagePath = '../app/locales/'.$this->_sLanguage.'.php';
            if ( file_exists ( $sLanguagePath ) )
            {
                include_once ("$sLanguagePath");
            }

            $someUrlParameters = parse_url("http://".$_SERVER['HTTP_HOST'].$p_sUrl);
            $this->_sUrl = $someUrlParameters['path'];

            $this->_sUrl = $this->checkPages($p_sUrl);

            // Step 1. Check if we are at the root app.
            if ($this->_sUrl == C_ROOT_KEY && array_key_exists(C_ROOT_KEY, $this->_someControllers)) {                
                if ($this->call(C_ROOT_KEY, $this->_someControllers[C_ROOT_KEY]) == true)
                    return true;
                else
                    return false;
            }

            // Search for an application to dispatch the request to.
            foreach ($this->_someControllers[$this->_sLanguage] as $sController => $someControlParam) {
                $sUrlCompare = $this->_sUrl."/";
                $sRegex = "|^/?". str_replace('*', '.*', $sController) . "$|";
                if (preg_match($sRegex, $sUrlCompare)) {
                    if ($this->call($sController, $someControlParam) == true)
                    {
                        return true;
                    }
                    else
                    {
                        return false;
                    }
                    //exit;
                }
            }

            // Finally, if we arrive here give up and display a 404 not found.
            $this->showError(C_ERROR_404ERROR);
        }
        catch(CException $e) {
            $this->manageLog($e);
        }

         return false;
    }

    private function _detectGetVars($p_sUrl)
    {
        try {
            
            
            
            $iPos = stripos($p_sUrl, "?");          
             
            //detect bank gateway parameters
             if (( $iPos !== false ) && (preg_match("/widgets/", $p_sUrl)))
             {
                 //if it finds the char ?
                 $sUrl = substr($p_sUrl, 0,$iPos);
                 
                 $oSession = new CAppSession();
                 
                 $sParamsUrl = substr($p_sUrl, $iPos+1);
                 $someUrlParams = explode("&",$sParamsUrl);
                 $someSessionValues = array();
                 foreach($someUrlParams as $sParam)
                 {
                     $someValues = explode("=", $sParam);
                     $someSessionValues[$someValues[0]] = $someValues[1]; 
                 }
                 $oSession->__setArray("someRequestParams", $someSessionValues);
                 
                 return $sUrl;
             }
             else {
                 return $p_sUrl;
             }
        }
        catch(CException $e) {
            throw $e;
        }        
    }


    public function encryptUrl($p_sUrl) {
        try
        {
            if(preg_match("/ajax/", $p_sUrl)) return str_replace( "ajax/", "", $p_sUrl );
            if(preg_match("/api/", $p_sUrl)) return $p_sUrl;
            if(preg_match("/crons/", $p_sUrl)) return $p_sUrl;
            if(preg_match("/widgets/", $p_sUrl)) return $p_sUrl;
            if(preg_match("/widgetsjs/", $p_sUrl)) return $p_sUrl;
            if(preg_match("/orders/", $p_sUrl)) return $p_sUrl;
            if(preg_match("/csvs/", $p_sUrl)) return $p_sUrl;
            if(preg_match("/calendars/", $p_sUrl)) return $p_sUrl;
            if(preg_match("/page/", $p_sUrl)) return $p_sUrl;            
            
            //TODO: change the previous preg_match by strstr because strstr is faster than preg_match.
            if(strstr($p_sUrl, "/agendas/duplicate/")) return $p_sUrl;
            if(strstr($p_sUrl, "/agendas/insertIntoUsersAgendas/")) return $p_sUrl;
            if(strstr($p_sUrl, "/timezones/timezone/")) return $p_sUrl;
            if(strstr($p_sUrl, "/payments/")) return $p_sUrl;
            if(strstr($p_sUrl, "/scripts/")) return $p_sUrl;
            if(strstr($p_sUrl, "/users/updateExpirationDate/")) return $p_sUrl;
            
            


            if( defined ( 'C_ENCRYPT' ) && C_ENCRYPT == 1 )
            {
                $oSession = new CAppSession();
                $p_sUrl = C_ROOT_KEY.$oSession->decrypt( str_replace( C_ROOT_KEY, "", $p_sUrl ) );
            }

            return $p_sUrl;
        }
        catch(CException $e) {
            $this->manageLog($e);
        }
    }

    /**
     * This function test and MAPS the passed parameters by the url to
     * GET parameters
     * @param array $p_someConfig array with parameter types
     * @param array $p_someValues array with parameter values
     * @return This funciton returns the number of mapped parameters.
     */
    private function _params($p_someConfig, $p_someValues) {
        try {
            $iReturnValue = 0;
           // $someTempGet = $_GET;
            unset($_GET);
            for ($i = 0, $l = count($p_someValues); $i < $l; $i++) {
//                if (!empty($p_someValues[$i])) {
                    $sValue = urldecode(trim($p_someValues[$i]));
                    $_GET[trim($i)] = $sValue;
                    $iReturnValue++;
//                }
            }

            //$_GET = array_merge($_GET, $someTempGet);

        }
        catch(CException $e) {
            throw $e;
        }
        return $iReturnValue;
    }

    /**
     * Extract the parameter values from the requested URL. These values are
     * mapped onto the parameter names.
     *
     * @param $p_sApplication the application URL mapping
     * @return an array of parameters.
     */
    private function _getValues($p_sApplication) {
        try {
            // Remove the wildcard from the end
            $sApplication = preg_replace('|/?\*$|', '', $p_sApplication);

            // Remove beginning and trailing slash from URL
            $sRequest = preg_replace('#^/|/$#', '', $this->_sUrl);

            // Strip the application URL to only have additional parameters left.
            // Preg-replace needed to avoid having an empty parameter (first slash)
            $sValues = preg_replace('|^'. $sApplication .'/?|', '', $sRequest);
        }
        catch(CException $e) {
            throw $e;
        }

        return explode('/', $sValues);
    }

    /**
     * @assert ("parama",array()) == C_ERROR_CONTROLLERNOTDEFINED
     * @assert ("parama/",array("controller" => "Mainpages")) == true
     * @assert ("",array()) == C_ERROR_CONTROLLERNOTDEFINED
     *
     * Dispatch the request to the responsible <code>Controller</code>.
     *
     * @param $p_sController the name of the application (for logging only).
     * @param $p_someControlParam index for the application configuration.
     */
    public function call($p_sController, $p_someControlParam) {
        try {

            $bReturnValue = true; //we will return false if function not success

            //we have to check if there are available controllers with that name                       
            if (!isset($p_someControlParam['controller'])) {
                return $this->showError(C_ERROR_CONTROLLERNOTDEFINED);
            } 

            //this function maps in $_GET global constant, the parameters passed by the url /parama/paramb
            $this->_params($p_someControlParam, $this->_getValues($p_sController));
            

            //Check is class is defined.
            //If it's not defined it could be because is a plugin Controller or an error.
            //Get plugins controller list
            $sNameController = C_CLASS_FILE . ucfirst(strtolower
                   (__search($p_someControlParam['controller'],$this->_sLanguage))) . C_CONTROLLER_CLASS . ".php";
            $sNameClass = C_CLASS_FILE . ucfirst(strtolower
                   (__search($p_someControlParam['controller'],$this->_sLanguage))) . C_CONTROLLER_CLASS;

            $sControllerPath = "";
            //loop that searches in the plugins directory. If a pluagin controller exists in this directory
            //we will use the new value for $sControllerPath
            if (!file_exists(C_CONTROLLERS . $sNameController))
            {
                $oDir = opendir(C_PLUGINS);

                while ($sPluginFolder = readdir($oDir))
                {
                    if(file_exists(C_PLUGINS . $sPluginFolder . "/controllers/" . $sNameController))
                    {
                        $sControllerPath = C_PLUGINS . $sPluginFolder . "/controllers/";
                        break;
                    }
                }
            }

            //we get an specific controller and instantiate it.
            require_once($sControllerPath . $sNameController);
            $oClass = basename($sNameClass);
            $oController = new $oClass();

            //set the language to controller
            $oController->setLanguage($this->_sLanguage, $this->_sLanguageDefault);

            $oController->getUsedModels();
            $oController->getControllers();

            //we have to check if the used controller is special for api.
            //if is an ApiController we check if C_API_ONOFF is set to FALSE because
            //then we don't have to attend this request

            if ($oController->getType() == C_API) {
                if (C_API_ONOFF == 0) {
                    return $this->showError(C_ERROR_APINOTCONNECTED);
                }
            }

            if (is_subclass_of($oController, 'CController')) {
                switch ($_SERVER['REQUEST_METHOD']) {
                    case 'GET':
                        $bReturnValue = $oController->renderVariablesGet();
                        break;
                    case 'POST':
                            $bReturnValue = $oController->renderVariablesPost();
                            break;
                    case 'DELETE':
                        if ($oController->getType() == C_API)
                            $bReturnValue = $oController->doDelete();
                        break;
                    case 'PUT':
                        if ($oController->getType() == C_API)
                            $bReturnValue = $oController->doPut();
                        break;

                    default:
                        return $this->showError(C_ERROR_REQUESTMETHOD);
                }
            }        
                 
            //if we get a 404 Error we redirect to 404.php
            if ($bReturnValue == false) {
                return $this->showError(C_ERROR_404ERROR);
            }
        }
        catch(CException $e) {
            throw $e;
        }
        return $bReturnValue;
    }

    /**    
     * Private method that loads a language.
     * @param <type> $p_sUrl we pass a complete url. This url will be like www.domain.com/es/actions. You
     * can see in the second position /es/ value. This value is the parameter that indicates the selected language.
     *
     * @return <type> this function returns a string with the replaced url without language.
     */
    private function selectLanguage($p_sUrl)
    {
       try {
            if(defined('C_LANGUAGES'))
            {
                $oSession = new CAppSession();
                //Get posible languages
                $someLanguages = explode(',',C_LANGUAGES);

                //Select default language
                if($oSession->__get("userproflanguage")){
                    $this->_sLanguage = $oSession->__get("userproflanguage");
                }
                else if(defined('C_LANGUAGE_DEFAULT'))
                {
                    $this->_sLanguage = C_LANGUAGE_DEFAULT;
                }
                else
                {
                    $this->_sLanguage = 'en';
                }

                //Search if there are any language in the url
                foreach($someLanguages as $sLanguage)
                {
                    if($this->matchLanguageString($sLanguage, $p_sUrl))
                    {
                        $this->_sLanguage = $sLanguage;
                        $this->_sLanguageDefault = false;
                        return str_replace(C_ROOT_KEY.$sLanguage."/", C_ROOT_KEY, $p_sUrl);
                    }
                }

                //else we return the complete url
                return $p_sUrl;
            }
            else
            {
                //Select default language
                if(defined('C_LANGUAGE_DEFAULT'))
                {
                    $this->_sLanguage = C_LANGUAGE_DEFAULT;
                }
                else
                {
                    $this->_sLanguage = 'en';
                }

                //we have to check if
                if($this->matchLanguageString($this->_sLanguage, $p_sUrl))
                {
                    $this->_sLanguageDefault = false;
                    return str_replace(C_ROOT_KEY.$this->_sLanguage."/", C_ROOT_KEY, $p_sUrl);
                }
                else
                {
                    return $p_sUrl;
                }
            }
       }
       catch(CException $e) {
           throw $e;
       }
    }

    /**
     * @assert("",C_ROOT_KEY."es/value") == false
     * @assert("es",C_ROOT_KEY."es/value") == true
     * @assert("en",C_ROOT_KEY."es/value") == false
     * 
     * This method compares two strings and return true if $p_sLanguage (es, en) is
     * included in the $p_sSubject (/bookitit/es/value). For example, the previous case
     * would return true.
     * @param <type> $p_sLanguage language to be searched
     * @param <type> $p_sSubject string with a language
     * @return <type> returns true if $p_sLanguage is included in $p_sSubject string
     */
    public function matchLanguageString($p_sLanguage, $p_sSubject)
    {
        try
        {
            $sLanguage = C_ROOT_KEY.$p_sLanguage."/";            
            $sLanguage = str_replace("/", "\/", $sLanguage);
            if(preg_match("/".$sLanguage."/", $p_sSubject))
            {
                return true;
            }            
        }
        catch(CException $e) {
            throw $e;
        }
        return false;
    }

    /**
     * @assert ("notexists") == "notexists"
     * 
     *
     * This function checks if the url is invoking a normal page,
     *
     * @param <type> $p_sUrl
     * @return <type>
     */
    public function checkPages($p_sUrl)
    {
        try
        {
            $sReturn = $p_sUrl;
            $sUrl = str_replace(C_ROOT_KEY, '', $p_sUrl);
            $sUrl = __search($sUrl, $this->_sLanguage);
            if(file_exists('../app/views/pages/'.$sUrl.'.php'))
            {
                $sReturn = C_ROOT_KEY.'page/'.$sUrl;
            }
        }
        catch(CException $e)
        {
            throw $e;
        }
        return $sReturn;
    }


    /**
     * This function show error view
     *
     * @param <mix> $p_sError Error Text and Integer
     * @return <type>
     */
    private function showError($p_sError)
    {
        try
        {
            redirectError( C_ERROR_VIEW , $p_sError , C_ERROR_SESSION);
        }
        catch(CException $e)
        {
            throw $e;
        }
        return $p_sError;
    }
    
    /**
     * This function manage Logs
     *
     * @param <mix> $p_sError Error Text and Integer
     * @return <type>
     */
    private function manageLog($e)
    {
        try
        {
            $oLogController = new CLogController();
            $sInserted = $oLogController->insertLog($e->getMessage()."// Linea:".$e->getLine(),$e->getFile(), $e->getTrace());

            if ($sInserted)
            {
                $this->showError($sInserted);
            }
        }
        catch(CException $e)
        {
            $this->showError( $e->getTrace() );
        }
        return true;
    }

    protected function get_someControllers() {
        try {
            return $this->_someControllers;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    protected function set_someControllers($p_SomeControllers) {
        try {
            $this->_someControllers = $p_SomeControllers;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    protected function set_sLanguage($p_sLanguage) {
        try {
            $this->_sLanguage = $p_sLanguage;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    protected function get_sLanguage() {
        try {
            return $this->_sLanguage;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    protected function get_sUrl() {
        try {
            return $this->_sUrl;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    protected function set_sUrl($p_sUrl) {
        try {
            $this->_sUrl = $p_sUrl;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    protected function get_sLanguageDefault() {
        try {
            return $this->_sLanguageDefault;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    protected function set_sLanguageDefault($p_sLanguageDefault) {
        try {
            $this->_sLanguageDefault =  $p_sLanguageDefault;
        }
        catch(CException $e) {
            throw $e;
        }
    }
}
?>
