<?php

/**
 * Class to be extended for all Models. This class give us some methods to access the database.
 * For example a CUser class -> CModel will have the methods ->save or ->delete, to his table. Besides
 * model classes will extend from this class.
 * @author Nubesis http://www.nubesis.com
 * @copyright Nubesis S.L.
 */

class CModel {
    /**
     * This attribute gets a list of rows called in select method. We will have a table with a hashtable.
     * like   [0] => Array
                   [nick] => companyA
                   [deleted] => 0
              [1] => Array
                   [nick] => companyB
                   [deleted] => 0
                   ....     
     * @protected
     */
    protected $_someData = array();

    /*the name of the table model
     *@protected
     */
    protected $_sTable = null;

    protected $_iIdLastInserted = -1;

    protected $_oController = null;

    protected $_sMethod = "";

    protected $_bTransaction = false;

    protected $_bBeginTransaction = false;
    
    /**
     * This field will have the validators for each class of the model.
     * '*' == all views
     * Examples:
     * $_someValidators = array(
     *              "view" => array(
     *                      "column1" => array(
     *                          C_VALID_NOT_EMPTY => array(
     *                              "message" => "This field can't be empty"
     *                          ),
     *                          C_VALID_MIN => array(
     *                              "min" => 3,
     *                              "message" => "Must be greater than 3"
     *                          )
     *                      ),
     *                      "column2" => array(
     *                          C_VALID_NOT_EMPTY => array(
     *                              "message" => "This field can't be empty"
     *                          )
     *                      )
     *                  )
     *              )
     */
    protected $_someValidators = array();

    /**
     * This field will have the ajax validators for each class of the model.
     * '*' == all views
     * Examples:
     * $_someAjaxValidators = array(
     *              "view" => array(
     *                      "column1" => array(
     *                          C_VALID_NOT_EMPTY => array(
     *                              "messageError" => "This field can't be empty"
     *                              "messageOk" => "This field it's OK"
     *                          ),
     *                          C_VALID_MIN => array(
     *                              "min" => 3,
     *                              "messageError" => "This field must be greater than 3"
     *                              "messageOk" => "This field it's OK"
     *                          )
     *                      ),
     *                      "column2" => array(
     *                          C_VALID_NOT_EMPTY => array(
     *                              "messageError" => "This field can't be empty"
     *                              "messageOk" => "This field it's OK"
     *                          )
     *                      )
     *                  )
     *              )
     */
    protected $_someAjaxValidators = array();

    protected $_someErrors = array();


    
    
    public function CModel() {

    }
    

    /*****************************/
    /********* BD ACTIONS ********/
    /*****************************/

    /**
     * @assert (array('fechahora' => '2010-05-13 00:00:00','descripcion' => 'error,'pantalla' => 'pantalla','usuario' => 'test@test.com','empresa' => 0,'trace' => 'someTrace')) == C_WARNING_INSERTEDOK
     * @assert (array('tablalogs' => array ('fechahora' => '2010-05-13 00:00:00','descripcion' => 'error,'pantalla' => 'pantalla','usuario' => 'test@test.com','empresa' => 0,'trace' => 'someTrace'))) == C_WARNING_INSERTEDOK
     * @assert (array(0 => array('fechahora' => '2010-05-13 00:00:00','descripcion' => 'error,'pantalla' => 'pantalla','usuario' => 'test@test.com','empresa' => 0,'trace' => 'someTrace'),1 => array('fechahora' => '2010-05-13 00:00:00','descripcion' => 'error,'pantalla' => 'pantalla','usuario' => 'test@test.com','empresa' => 0,'trace' => 'someTrace'))= == C_WARNING_INSERTEDOK
     * @assert (array('tablalogs' => array (0 => array('fechahora' => '2010-05-13 00:00:00','descripcion' => 'error,'pantalla' => 'pantalla','usuario' => 'test@test.com','empresa' => 0,'trace' => 'someTrace'),1 => array('fechahora' => '2010-05-13 00:00:00','descripcion' => 'error,'pantalla' => 'pantalla','usuario' => 'test@test.com','empresa' => 0,'trace' => 'someTrace')))= == C_WARNING_INSERTEDOK
     *
     * This function inserts stored data in the database.
     * @param <array> $p_someValues array with data. array('tablename' => array(0 => 'column' => value)).
     * @param <type> $p_bDefaultTable On one hand, if it's false, the software will not use the parameter
     * assigned in $this->_sTable, and you will have to pass a parameter in the first position of the array of $p_someValues
     * to indicate the software which table will have to use.
     * On the other hand if it's true the system will use the parameter assigned in the attribute
     * $this->_sTable of the class of CModel
     * @return <int>
     */

    public function insert($p_someValues , $p_bDefaultTable = true, 
            $p_bEvaluateValidates = false) {
        try {

            $bUpdatedColumnExist = false;
            $bCreatedColumnExist = false;

            $bUpdatedValueExist = false;
            $bCreatedValueExist = false;

            $bReturnValidates = $this->validates($p_someValues, $p_bDefaultTable, $p_bEvaluateValidates);
            if ($bReturnValidates == true)
            {
                return C_ERROR_VALIDATORS;
            }


            $bSingleInsert = true;


            if($p_bDefaultTable)
            {
                if($this->_sTable != null)
                {
                    $someInsert = array($this->_sTable => $p_someValues);
                    $p_someValues = $someInsert;
                }
                else
                {
                    return C_ERROR_GENERAL;
                }
            }
            $iReturnValue = C_ERROR_EXECSQL;

            $oCon = CAppConnectionManager::get();
            $oDb = $oCon->getConnection();

            if(!$oDb)
            {
                throw new CException("Error: ".mysql_errno($oDb).':'.mysql_error($oDb),C_ERROR_NOCONNECTION);
            }
            else
            {
                //select database
                $oConnectionOk = $oCon->selectDataBase();

                foreach ($p_someValues as $sNameTable => $someTableValues):

                    //Get Column description
                    $this->describe( $sNameTable );

                    foreach( $this->_someInfoData as $oDescribe)
                    {
                        if( $oDescribe["Field"] == 'created')
                            $bCreatedColumnExist = true;
                        if( $oDescribe["Field"] == 'updated')
                            $bUpdatedColumnExist = true;
                    }

                    $sQuery = "INSERT INTO ".$sNameTable." ";

                    //For multiple inserts
                    $sAllValues = "";

                    //For single insert
                    $sValues = "";
                    $sColumns = "";

                    foreach ($someTableValues as $someIndex => $someValues):

                        if(is_array($someValues))
                        {
                            $bSingleInsert = false;
                            //Multiple insert
                            $sAllColumns = "";

                            $sAllValues .= "(";
                            $sAllColumns .= "(";

                            foreach ($someValues as $column => $value):
                                $sType = $this->get_sFieldType($column);

                                if( $sType == "string" ){
                                    $sAllValues .= "'$value',";
                                }
                                else if( $sType == "number" && ( is_null($value) || strcasecmp($value, "null") == 0 || strlen(trim($value)) <= 0 ) ){
                                    $sAllValues .= "null,";
                                }
                                else{
                                    $sAllValues .= "$value,";
                                }

                                $sAllColumns .= "$column,";

                            if( $column == 'created')
                                $bCreatedValueExist = true;
                            if( $column == 'updated')
                                $bUpdatedValueExist = true;

                            endforeach;

                            if( $bCreatedColumnExist && !$bCreatedValueExist )
                            {
                                $sDate = date("Y-m-d H:i:s");
                                $sAllValues .= "'$sDate',";
                                $sAllColumns .= "created,";
                            }
                            
                            if( $bUpdatedColumnExist && !$bUpdatedValueExist )
                            {
                                $sDate = date("Y-m-d H:i:s");
                                $sAllValues .= "'$sDate',";
                                $sAllColumns .= "updated,";
                            }

                            $sAllValues = substr($sAllValues, 0, -1);
                            $sAllValues .= "),";

                            $sAllColumns = substr($sAllColumns, 0, -1);
                            $sAllColumns .= ") VALUES ";
                        }
                        else
                        {
                            $sType = $this->get_sFieldType($someIndex);

                            if( $sType == "string" ){
                                $sValues .= "'$someValues',";
                            }
                            else if( $sType == "number" && ( is_null($someValues) || strcasecmp($someValues, "null") == 0 || strlen(trim($someValues)) <= 0 ) ){
                                $sValues .= "null,";
                            }
                            else{
                                $sValues .= "$someValues,";
                            }

                            $sColumns .= "$someIndex,";

                            if( $someIndex == 'created')
                                $bCreatedValueExist = true;
                            if( $someIndex == 'updated')
                                $bUpdatedValueExist = true;
                        }

                    endforeach;
                    if($bSingleInsert)
                    {
                        if( $bCreatedColumnExist && !$bCreatedValueExist )
                        {
                            $sDate = date("Y-m-d H:i:s");
                            $sValues .= "'$sDate',";
                            $sColumns .= "created,";
                        }

                        if( $bUpdatedColumnExist && !$bUpdatedValueExist )
                        {
                            $sDate = date("Y-m-d H:i:s");
                            $sValues .= "'$sDate',";
                            $sColumns .= "updated,";
                        }

                        //Single insert
                        $sColumns = "(".substr($sColumns, 0, -1).") VALUES ";

                        $sValues = "(".substr($sValues, 0, -1).");";

                        $sQuery .= "$sColumns $sValues";
                    }
                    else
                    {
                        $sQuery .= $sAllColumns." ".substr($sAllValues, 0 , -1).";";
                    }

                    if($this->_bTransaction) $this->beginTransaction();

                    $oResult = mysql_query($sQuery);

                    $this->_iIdLastInserted = mysql_insert_id();

                    if($oResult==FALSE){
                        $this->rollBackTransaction();
                            $iReturnValue = C_ERROR_EXECSQL;

                    }else{
                            $iReturnValue = C_WARNING_INSERTEDOK;

                    }
                endforeach;
            }
        }
	catch (CException $e) {
            throw $e;
	}
        return $iReturnValue;
    }
    
    
    public function query($p_sQuery) {
        try {
            $oCon = CAppConnectionManager::get();
            $oDb = $oCon->getConnection();

            if(!$oDb)
            {
                throw new CException("Error: ".mysql_errno($oDb).':'.mysql_error($oDb),C_ERROR_NOCONNECTION);
            }
            else
            {
                //select database
                $oConnectionOk = $oCon->selectDataBase();
                $oResult = mysql_query($p_sQuery);
                $this->_iIdLastInserted = mysql_insert_id();
                if($oResult==FALSE){
                    return C_ERROR_EXECSQL;
                }
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return true;
    }
    

     /**
      * @assert ("empresa = 1", C_TABLE_ERROR_LOG) == C_WARNING_UPDATEDOK
      * @assert ("empresa = 1, usuario = 'dos'", C_TABLE_ERROR_LOG, "usuario like '%prueba@xavo%'") == C_WARNING_UPDATEDOK
      * @assert ("", C_TABLE_ERROR_LOG) == C_ERROR_EXECSQL
      *
      * This function executes an update in the database. You have to pass the correct parameters to the function.
      * @param <type> $p_someColumns It's an array with the name of the columns that we want to update.
      * @param <type> $p_someValues It's an array with the values for each of the previous columns.
      * @param <type> $p_sTable The table where we want to update
      * @param <type> $p_sWhere The where condition. What parameters will be involved.
      * @return <type> It returns C_WARNING_UPDATEDOK if the execution success, or it will return C_ERROR_EXECSQL if something was wrong.
      */
    public function update($p_someUpdate, $p_sWhere = null, $p_bDefaultTable = true,
            $p_bEvaluateValidates = false) {
        try {
            $bUpdatedColumnExist = false;

            $bUpdatedValueExist = false;

            $bReturnValidates = $this->validates($p_someUpdate, $p_bDefaultTable, $p_bEvaluateValidates);
            if ($bReturnValidates == true)
            {
                return C_ERROR_VALIDATORS;
            }

            $iReturnValue = C_ERROR_EXECSQL;

            $oCon = CAppConnectionManager::get();
            $oDb = $oCon->getConnection();

            if(!$oDb)
            {
                throw new CException("Error: ".mysql_errno($oDb).':'.mysql_error($oDb),C_ERROR_NOCONNECTION);
            }
            else
            {
               //Check if table exist
                if($p_bDefaultTable)
                {
                    if($this->_sTable != null)
                    {
                        $someUpdate = array($this->_sTable => $p_someUpdate);
                        $p_someUpdate = $someUpdate;                             
                    }
                    else
                    {
                        return C_ERROR_GENERAL;
                    }
                }
                
                //select database
                $oCon->selectDataBase();

                foreach ($p_someUpdate as $sNameTable => $someTableValues):

                    //Get Column description
                    $this->describe( $sNameTable );

                    foreach( $this->_someInfoData as $oDescribe)
                    {
                        if( $oDescribe["Field"] == 'updated')
                            $bUpdatedColumnExist = true;
                    }
                
                    $sQuery = "UPDATE ".$sNameTable." ";

                    //For single insert
                    $sValues = "";

                    foreach ($someTableValues as $someIndex => $someValues):

                        if($someIndex != 'id'.$sNameTable)
                        {
                             if(is_array($someValues))
                             {
                                    //Multiple insert                                    
                                    $sAllValues = "";

                                    foreach ($someValues as $column => $value):
                                        if( $this->get_sFieldType($column) == "string" )
                                            $sAllValues .= "$column = '$value',";
                                        else
                                            $sAllValues .= "$column = $value,";

                                        if( $column == 'updated')
                                            $bUpdatedValueExist = true;
                                    endforeach;                             

                                    $sValues = $sAllValues;
                                    
                            }
                            elseif($this->get_sFieldType($someIndex) == 'string')
                            {
                                if( $someIndex == 'updated')
                                    $bUpdatedValueExist = true;
                                
                                $sValues .= "$someIndex = '$someValues',";
                            }
                            else
                            {
                                if( $someIndex == 'updated')
                                    $bUpdatedValueExist = true;

                                if( !is_numeric($someValues) )
                                {
                                    if($someValues != "")
                                        $sValues .= "$someIndex = $someValues,";
                                }
                                else
                                {
                                    $sValues .= "$someIndex = $someValues,";
                                }

                            }
                        }

                    endforeach;

                    if( $bUpdatedColumnExist && !$bUpdatedValueExist )
                    {
                        $sDate = date("Y-m-d H:i:s");
                        $sValues .= "updated = '$sDate',";
                    }

                    $sValues = "SET ".substr($sValues, 0, -1);

                    if (($p_sWhere == "") || ($p_sWhere == null))
                        $sQuery .= "$sValues;";
                    else
                        $sQuery .= "$sValues  WHERE $p_sWhere;";

                    if($this->_bTransaction) $this->beginTransaction();
                    
                    $oResult = mysql_query($sQuery);

                    if($oResult==FALSE){
                        $iReturnValue = C_ERROR_EXECSQL;
                    }else{
                        $iReturnValue = C_WARNING_UPDATEDOK;
                    }
                endforeach;
            }
        }
	catch (CException $e) {
            throw $e;
	}
        return $iReturnValue;
    }

    /**
     * @assert (C_TABLE_ERROR_LOG, "") == C_WARNING_UPDATEDOK
     * @assert (C_TABLE_ERROR_LOG) == C_WARNING_UPDATEDOK
     * @assert (C_TABLE_ERROR_LOG, "empresa = 1") == C_WARNING_UPDATEDOK
     * @param <type> $p_sTable Table database that we want to delete.
     * @param <type> $p_sWhere The condition to delete the data values.
     * @return <type> It returns C_ERROR_EXECSQL when function fails and return C_WARNING_DELETEOK when
     */
    public function delete($p_sTable = null, $p_sWhere = null, $p_isLogic = false )
    {
        try
        {
            $iReturnValue = C_ERROR_EXECSQL;
            $oCon = CAppConnectionManager::get();
            $oDb = $oCon->getConnection();

            if(!$oDb)
            {
                throw new CException("Error: ".mysql_errno($oDb).':'.mysql_error($oDb),C_ERROR_NOCONNECTION);
            }
            else
            {
               //Check if table exist
                if($p_sTable == null)
                {
                    if($this->_sTable != null)
                    {
                        $p_sTable = $this->_sTable;
                    }
                    else
                    {
                        return C_ERROR_GENERAL;
                    }
                }

                //select database
                $oCon->selectDataBase();

                if (($p_sWhere == "") || ($p_sWhere == null))
                {
                    if($p_isLogic)
                        $sQuery = "UPDATE ".$p_sTable ." SET deleted = 1, updated = '" . $sDate = date("Y-m-d H:i:s") . "' ";
                    else
                  $sQuery = "DELETE FROM ".$p_sTable;
                }
                else
                {
                    if($p_isLogic)
                        $sQuery = "UPDATE ".$p_sTable . " SET deleted = 1, updated = '" . $sDate = date("Y-m-d H:i:s") . "' WHERE ". $p_sWhere;
                    else
                  $sQuery = "DELETE FROM ".$p_sTable." WHERE ".$p_sWhere;
                }

                if($this->_bTransaction) $this->beginTransaction();
                
                $oResult = mysql_query($sQuery);

                if($oResult==FALSE)
                {
                  $iReturnValue = C_ERROR_EXECSQL;
                }
                else
                {
                  $iReturnValue = C_WARNING_DELETEOK;
                }
            }
        }
        catch (CException $e)
        {
            throw $e;
        }
        return $iReturnValue;
    }

    /**
     * @assert ("*","descripcion like '%test%'", null, null, null, null, C_TABLE_ERROR_LOG) == C_WARNING_SELECTEDOK
     * @assert ("*","descripcion like '%test%'", null, null, null, "LIMIT 0,10", C_TABLE_ERROR_LOG) == C_WARNING_SELECTEDOK
     * @assert ("",null, null, null, null, null, C_TABLE_ERROR_LOG) == C_ERROR_EXECSQL
     * This function retrieves values from the database. In the parameters you will have to complete the sentences
     * @param <array> $p_someValues = array(
                "p_sColumns" => null,
                "p_sWhere" => null,
                "p_sOrder" => null,
                "p_sDirection" => null,
                "p_sGroup" => null,
                "p_sLimit" => null,
                "p_sTable" => null ,
                "p_sPagination" => false
            );
     * @param <type> $p_sColumn select the columns that you want retrieve. *, name, etc
     * @param <type> $p_sWhere you have to select the conditions to retrieve the values
     * @param <type> $p_sOrder you can say, the order of the results order by
     * @param <type> $p_sDirection you can say, the order of the results order by
     * @param <type> $p_sGroup if you want, you can group the results
     * @param <type> $p_sLimit you can establish a number of results
     * @param <type> $p_sTable the table with the values that you want get
     * 
     * @return <type>
     */
    public function select($p_someValues)
    {
        try {
            //Init values
            $this->_someData = array();
            $p_sColumns = null;
            $p_sWhere = null;
            $p_sOrder = null;
            $p_sDirection = null;
            $p_sGroup = null;
            $p_sLimit = null;
            $p_sTable = null;
            $p_bPagination = false;

            extract($p_someValues);
            
            $iReturnValue = C_ERROR_EXECSQL;
            if ((!isset($p_sColumns)) || ($p_sColumns == "")) return $iReturnValue;

            $oCon = CAppConnectionManager::get();
            $oDb = $oCon->getConnection();

            if(!$oDb)
            {
                throw new CException("Error: ".mysql_errno($oDb).':'.mysql_error($oDb),C_ERROR_NOCONNECTION);
            }
            else
            {
                $sWhere = null;
                $sOrder = null;
                $sGroup = null;
                $sColumns = '*';
                $sPage = 0;
                $sDirection = "ASC";

                if($p_bPagination)
                    $this->get_paginateValues($sPage, $sOrder, $sDirection, $p_sLimit);

                //Check if table exist
                if($p_sTable == null)
                {
                    if($this->_sTable != null)
                    {
                        $p_sTable = $this->_sTable;
                    }
                    else
                    {
                         return C_ERROR_GENERAL;
                    }
                }

                //Check if columns clausule exists
                if($p_sColumns != null)
                {
                    $sColumns = $p_sColumns;
                }

                //Check if where clausule exists
                if($p_sWhere != null)
                {
                    $sWhere = 'WHERE ';
                    $sWhere .= $p_sWhere;
                }

                //Check if order clausule exists
                //if we have passed a paramter with an order is != null
                if($p_sOrder != null)
                {
                    //if you are using the pagination we will use $sOrder value
                    if($sOrder != null)
                    {
                        $sOrder = "ORDER BY $sOrder";
                    }
                    else //else we have to continue using the parameter passed
                    {
                        $sOrder = "ORDER BY $p_sOrder";
                    }
                }
                elseif($sOrder != null) //if we haven't passed a parameter and we are using pagination
                {
                    $sOrder = "ORDER BY $sOrder";
                }

                //Check if group clausule exists
                //if we have passed a paramter with an group is != null
                if($p_sGroup != null)
                {
                    $sGroup = "GROUP BY $p_sGroup";
                }

                //Check if direction clausule exists
                if($p_sDirection != null)
                {
                    if($sDirection == "ASC")
                        $sDirection = $p_sDirection;
                }
                if($sDirection == "ASC")
                    $sDirection = $p_sDirection;
                
                //Check if limit clausule exists
                $sLimit = "";
                if($p_bPagination)
                {
                    if($p_sLimit != null)
                        $sLimit = "limit $sPage,$p_sLimit";
                    else
                        $sLimit = "limit $sPage,10";
                }
                else
                {
                    if($p_sLimit != null)
                        $sLimit = "limit 0,$p_sLimit";
                }

                //select database
                $oCon->selectDataBase();
		$sQuery = "SELECT $sColumns FROM $p_sTable $sWhere $sGroup $sOrder $sDirection $sLimit;";

		$oResult = mysql_query($sQuery);
                
		if ($oResult == FALSE) {
                    $iReturnValue = C_ERROR_EXECSQL;
		}
                else {
                    if (mysql_num_rows($oResult))
                    {
                        $this->_someData = array();
                        while($oRow = mysql_fetch_assoc($oResult))
                        {
                            $this->_someData[] = $oRow;
                        }
                    }
                    $iReturnValue = C_WARNING_SELECTEDOK;
                }
            }
        }
	catch (CException $e) {
            throw $e;
	}

        return $iReturnValue;
    }

    /**
     * This function validates each parameter passed by post or get by the user through
     * a form. If there is errors we will have to show them to the user.
     * @param <type> $p_someValues parameters of the user
     * @param <type> $p_bDefaultTable On one hand, if it's false the software will not use the parameter
     * assigned in $this->_sTable, and you will have to pass it in the first position of the array of $p_someValues.
     * On the other hand if it's true the system will use the parameter assigned in the attribute
     * $this->_sTable of the class of CModel
     * @return <type> this function returns true if there is no errors or false on errors
     */
    public function validates($p_someValues, $p_bDefaultTable = true, $p_bEvaluateValidates = false) {
        try {
            if ( count( $this->_someValidators ) <= 0 || $p_bEvaluateValidates == false) {
                return false;
            }

            $bError = false;
            if ($p_bDefaultTable == false) {
                foreach($p_someValues as $someValues) {
                    $p_someValues = $someValues;
                    break;
                }
            }

            //first, we get the erros and validators
            $someValidators = $this->get_someValidators();

            //for each value we have to check all the validators.
            foreach ($p_someValues as $someIndexModel => $someValuesAttributes)
            {
                foreach ($someValuesAttributes as $someIndexValueAtt => $oValueAtt)
                {
                    //we get each validation for a concrete validator
                    if (isset($someValidators["*"][$someIndexValueAtt]))
                        $someValidations = $someValidators["*"][$someIndexValueAtt];
                    else
                        continue;
                    foreach($someValidations as $sFunction => $oValidation) {
                        $bValue = $this->$sFunction( $someIndexValueAtt , $oValueAtt , $oValidation );

                        if (!$bError)
                            $bError = $bValue;
                        
                        continue;
                    }
                }

                foreach ($someValuesAttributes as $someIndexValueAtt => $oValueAtt)
                {
                    //we get each validation for a concrete validator
                    if (isset($someValidators[$this->_sMethod][$someIndexValueAtt]))
                        $someValidations = $someValidators[$this->_sMethod][$someIndexValueAtt];
                    else
                        continue;
                    foreach($someValidations as $sFunction => $oValidation) {
                        $bValue = $this->$sFunction( $someIndexValueAtt , $oValueAtt , $oValidation );

                        if (!$bError)
                            $bError = $bValue;

                        continue;
                    }
                }
            }

            //only when the coder will pass a control parent
            if ($this->_oController != null) {
                $this->_oController->set_someErrorMessages($this->get_someErrors());
                $this->_oController->set_iTypeMessage(C_MESSAGE_WARNING);
            }

        }
        catch(CException $e) {
            throw $e;
        }
        return $bError;
    }

    public function describe($p_sTable = null)
    {
        $sQuery = "DESCRIBE $p_sTable ;";
        $iReturnValue = C_ERROR_EXECSQL;
        $oCon = CAppConnectionManager::get();
        $oDb = $oCon->getConnection();

        if(!$oDb)
        {
            throw new CException("Error: ".mysql_errno($oDb).':'.mysql_error($oDb),C_ERROR_NOCONNECTION);
        }
        else
        {
            $oConnectionOk = $oCon->selectDataBase();
            $oResult = mysql_query($sQuery);
            
            if($oResult==FALSE){
                $iReturnValue = C_ERROR_EXECSQL;
            }else{
                if (mysql_num_rows($oResult))
                {
                    $this->_someInfoData = array();

                    while($oRow = mysql_fetch_assoc($oResult))
                    {
                        $this->_someInfoData[] = $oRow;
                    }
                }

                $iReturnValue = C_WARNING_DESCRIBEOK;
            }
        }

        return $iReturnValue;
    }

    public function describeColumns($p_sTable = null,$p_bAutoInc = false) {
        $this->describe($p_sTable);
        $someFields = array();
        foreach($this->_someInfoData as $oSomeInfoData) {
            //i(f you want to show auto_increment fields you must change this line
            if ($p_bAutoInc == false) {
                if (isset($oSomeInfoData["Extra"]) && (strcasecmp($oSomeInfoData["Extra"],"auto_increment") != 0)) {
                    $someFields[] = $oSomeInfoData["Field"];
                }
            }
            else {
                    $someFields[] = $oSomeInfoData["Field"];
            }
        }

        return $someFields;
    }

    /*****************************/
    /***** GETTERS & SETTERS *****/
    /*****************************/

    public function get_someValidators() {
        try {
            return $this->_someValidators;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function get_someAjaxValidators() {
        try {
            return $this->_someAjaxValidators;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function set_someValidators($p_someValidators) {
        try {
            $this->_someValidators = $p_someValidators;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function set_someAjaxValidators($p_someAjaxValidators) {
        try {
            $this->_someAjaxValidators = $p_someAjaxValidators;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function get_someErrors() {
        try {
            return $this->_someErrors;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function set_someErrors($p_someErrors) {
        try {
            $this->_someErrors = $p_someErrors;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function setController($p_oController) {
        try {
            $this->_oController = $p_oController;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    /**
     * This function gets the array of the attribute _someData
     * @return array It's an array with the data values of the attribute _someData
     */
    public function get_someData() {
        return $this->_someData;
    }

    public function get_sFieldType($p_sColumn)
    {
        foreach ($this->_someInfoData as $someInfo):
            if($someInfo['Field'] == $p_sColumn)
            {
                $sType = explode('(', $someInfo['Type']);
                if($sType[0] == 'varchar' || $sType[0] == 'text' || $sType[0] == 'timestamp' || $sType[0] == 'date' || $sType[0] == 'datetime')
                {
                    return 'string';
                }
                else
                {
                    return 'number';
                }
            }
        endforeach;

        return false;
    }

    public function get_acceptFieldNull($p_sColumn)
    {
        foreach ($this->_someInfoData as $someInfo):
            if($someInfo['Field'] == $p_sColumn)
                return $someInfo['Null'];
        endforeach;

        return false;
    }

    public function get_iIdLastInserted() {
        try {
            return $this->_iIdLastInserted;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function get_paginateValues(&$sPage, &$sOrder, &$sDirection, $p_sLimit = null)
    {
        //Select paginator variables
        foreach ($_GET as $someGet):
            if(preg_match('/^page/', $someGet))
            {
                $somePage = explode(":", $someGet);

                if($p_sLimit == null)
                {
                    $p_sLimit = "10";
                    $sPage = ($somePage[1] - 1) * 10;
                }
                else
                {
                    $sPage = ($somePage[1] - 1) * $p_sLimit;
                }
            }

            if(preg_match('/^sort/', $someGet))
            {
                $someSort = explode(":",$someGet);
                $sOrder = $someSort[1];
            }

            if(preg_match('/^direction/', $someGet))
            {
                $someDirection = explode(":",$someGet);
                $sDirection = $someDirection[1];
            }
        endforeach;
    }

    /**
     * This function gets _sTable attribute
     * @return String name with the associated table model
     */
    public function get_sTable() {
        return $this->_sTable;
    }

    public function set_sTable($p_sTable) {
        $this->_sTable = $p_sTable;
    }

    public function setActiveMethod( $p_sMethod )
    {
        $this->_sMethod = $p_sMethod;
    }

    /*****************************/
    /******* TRANSACTIONS ********/
    /*****************************/
    public function startTransaction() {
        try {
            $this->_bTransaction = true;
            $this->_bBeginTransaction = true;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function beginTransaction() {
        try {
            if($this->_bBeginTransaction)
            {
                mysql_query("BEGIN");
                $this->_bBeginTransaction = false;
            }

        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function endTransaction() {
        try {
            mysql_query("COMMIT");
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function rollBackTransaction() {
        try {
            mysql_query("ROLLBACK");
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function getTotalRows()
   {
       try
       {
           $oCon = CAppConnectionManager::get();
           $oDb = $oCon->getConnection();

           if(!$oDb)
           {
               throw new CException("Error: ".mysql_errno($oDb).':'.mysql_error($oDb),C_ERROR_NOCONNECTION);
           }
           else
           {
               //select database
               $oConnectionOk = $oCon->selectDataBase();
               $someRows = mysql_fetch_row(mysql_query("SELECT FOUND_ROWS()"));

               if (isset($someRows[0])) return $someRows[0];
           }
       }
       catch(CException $e)
       {
           throw $e;
       }
       return -1;
   }
   
   /**
    * This function will check if a table exists or not
    * @param type $p_sName the name of the table to be checked
    * @return if the table exists will return true otherwise will return false
    */
   public function existsTable($p_sName)
   {
       try
       {
           $oCon = CAppConnectionManager::get();
           $oDb = $oCon->getConnection();

           if(!$oDb)
           {
               throw new CException("Error: ".mysql_errno($oDb).':'.mysql_error($oDb),C_ERROR_NOCONNECTION);
           }
           else
           {
               //select database
               $oConnectionOk = $oCon->selectDataBase();
               $someRows = mysql_fetch_row(mysql_query('SHOW TABLES LIKE "'.$p_sName.'"'));

               if (isset($someRows[0])) return true;
           }
       }
       catch(CException $e)
       {
           throw $e;
       }
       return false;
   }

    /*****************************/
    /********* VALIDATORS ********/
    /*****************************/
}
?>
