<?php
/**
 * CRestRequest
 * @author Nubesis http://www.nubesis.com
 * @copyright Nubesis S.L.
 */
class CRestUtils {

    public function CRestUtils() {
       
    }


    /**
     *
     * This function is called from a CApiController. The function will
     * return the data values passed by parameter in the correct format. XML, JSON or VXML
     *
     * @param <type> $p_someParameters 
     */
    public function writeData($p_someParameters) {
        $sString = "-1";
        try {
            if (strpos($_SERVER['HTTP_ACCEPT'], 'json')) {
               $sString = $this->writeToJson($p_someParameters);
            }
            elseif (strpos($_SERVER['HTTP_ACCEPT'], 'xml')) {
               $sString = $this->writeToXML($p_someParameters);
            }
            else {
               $sString = $this->writeToVXML($p_someParameters);
            }
        }
        catch(CException $e) {
            throw $e;                
        }

        return $sString;
    }

    /**
     * This function returns the values in a xml format
     * @param <type> $p_someParameters array with arrays of values to include in the xml
     * @param <type> $p_oRaiz root indicates if it's the root node. Else the function will run in recursive mode.
     * @return <type> a String with a piece of well xml formed
     */
    public function writeToXML($p_someParameters, $p_oRoot=-1){
        try {

            $sString = "";
            if ($p_oRoot == -1) $sString .= "<?xml version='1.0' encoding='utf-8'?>";
        
            foreach($p_someParameters as $sKey => $oValue)
            {
                if (is_array($oValue))
                {
                    if (($p_oRoot != -1) && !is_numeric($p_oRoot)) $sAux = substr($p_oRoot, 0, strlen($p_oRoot)-1);
                    else $sAux = $sKey;
                    $sString .= "<".$sAux.">";
                    $sString .= CRestUtils::writeToXML($oValue, $sKey);
                    $sString .= "</".$sAux.">";
                    continue;
                }
                else
                {
                    if (is_numeric($sKey)) $sKey = substr($p_oRoot, 0, strlen($p_oRoot)-1);
                    $sString .= "<".$sKey.">";
                    //@TODO: probably here we will have to eliminate special characters
                    $sString .= str_replace("_", " ", $oValue);
                    $sString .= "</".$sKey.">";
                    // add single node.
                    
                }
            }        
        }
        catch(CException $e) {
            throw $e;
        }
        
        return $sString;


    }

    /**
     * this function returns a String with $p_somPerameters in JSON format
     * @param <type> $p_someParameters
     * @return <type>
     */
    public function writeToJson($p_someParameters) {
        $sString = "";
        try {
            $sString = json_encode($p_someParameters);
        }
        catch(CException $e) {
            throw $e;
        }
        return $sString;
    }


    /**
     * This function returns a String with $p_someParameters in VXML format
     * @param <type> $p_someParameters values to return
     * @return <type> String with VXML format
     */
    public function writeToVXML($p_someParameters) {
        $sString = "";
        try {

        }
        catch(CException $e) {
            throw $e;
        }
        return $sString;
    }

    /**
     * This function Parses the Digest passed by http
     * @param <type> $p_sStringParse is a string to parse witha a digest of the values. The code will map in a vector to memory.
     * @return <type> return an array with the values of the Digest orderes by $needed_parts tags
     */
    public function httpDigestParse($p_sStringParse)
    {
        try {
            // protect against missing data
            $someNeededParts = array('nonce'=>1, 'nc'=>1, 'cnonce'=>1, 'qop'=>1, 'username'=>1, 'uri'=>1, 'response'=>1);
            $someData = array();
            $someKeys = implode('|', array_keys($someNeededParts));

            preg_match_all('@(' . $someKeys . ')=(?:([\'"])([^\2]+?)\2|([^\s,]+))@', $p_sStringParse, $someMatches, PREG_SET_ORDER);

            foreach ($someMatches as $m) {
                $someData[$m[1]] = $m[3] ? $m[3] : $m[4];
                unset($someNeededParts[$m[1]]);
            }
        }
        catch(CException $e) {
            throw $e;
        }

        return $someNeededParts ? false : $someData;
    }


    /**
     *
     * @param <type> $status
     * @return <type>
     */
    public function getStatusCodeMessage($status)
    {
        try {
                
		$someCodes = Array(
		    100 => 'Continue',
		    101 => 'Switching Protocols',
		    200 => 'OK',
		    201 => 'Created',
		    202 => 'Accepted',
		    203 => 'Non-Authoritative Information',
		    204 => 'No Content',
		    205 => 'Reset Content',
		    206 => 'Partial Content',
		    300 => 'Multiple Choices',
		    301 => 'Moved Permanently',
		    302 => 'Found',
		    303 => 'See Other',
		    304 => 'Not Modified',
		    305 => 'Use Proxy',
		    306 => '(Unused)',
		    307 => 'Temporary Redirect',
		    400 => 'Bad Request',
		    401 => 'Unauthorized',
		    402 => 'Payment Required',
		    403 => 'Forbidden',
		    404 => 'Not Found',
		    405 => 'Method Not Allowed',
		    406 => 'Not Acceptable',
		    407 => 'Proxy Authentication Required',
		    408 => 'Request Timeout',
		    409 => 'Conflict',
		    410 => 'Gone',
		    411 => 'Length Required',
		    412 => 'Precondition Failed',
		    413 => 'Request Entity Too Large',
		    414 => 'Request-URI Too Long',
		    415 => 'Unsupported Media Type',
		    416 => 'Requested Range Not Satisfiable',
		    417 => 'Expectation Failed',
		    500 => 'Internal Server Error',
		    501 => 'Not Implemented',
		    502 => 'Bad Gateway',
		    503 => 'Service Unavailable',
		    504 => 'Gateway Timeout',
		    505 => 'HTTP Version Not Supported'
		);

        
            return (isset($someCodes[$status])) ? $someCodes[$status] : '';
        }
        catch(CException $e) {
            throw $e;
        }

    }
}
?>
