<?php
/**
 * CView is the Base class to the rest of the views 
 * @author Nubesis http://www.nubesis.com
 * @copyright Nubesis S.L.
 */

class CView {

    /**
     * Variables passed from the controller up to the view.
     * @protected
     */
    protected $_someVars = array();

    /**
     * Name of view to show.
     * @protected
     */
    protected $_sView = '';

    /**
     * Path of the view.
     * @protected
     */
    protected $_sPath = '';

    /**
     * Parent controller used to call this view.
     * @protected
     */
    protected $_sParentController = '';

    /**
     * The language selected.
     * English default
     * @protected
     */
    protected $_sLanguage = 'en';
    
    /**
     * Indicates if the language has passed by the url
     * true default = not passed
     * @protected
     */
    protected $_sLanguageDefault = true;
    /**
     * Constructor for CView
     * @param String $p_sFile name of the file
     * @param String $p_sPath route to the file
     */
    public function CView($p_sFile, $p_sPath = C_VIEWS ) {
        try {
            $this->_sView = $p_sFile;
            $this->_sPath = $p_sPath;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    /**
     * this function adds a view variable.
     *
     * @param $p_sName the name of the view
     * @param $p_sValue the value of the variable
     */
    public function add($p_sName, $p_sValue) {
        try {
            //we have added isobject because in version 5.3.0, the function getclass has a bug
            if (is_object($p_sValue)){
                $this->_someVars[$p_sName] = get_class($p_sValue) == get_class($this) ?
                $p_sValue->fetch() : $p_sValue;
            }
            else {
                $this->_someVars[$p_sName] = $p_sValue;
            }
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function remove($p_sName) {
        try {
            if (isset($this->_someVars[$p_sName])) {
                unset($this->_someVars[$p_sName]);
            }
        }
        catch(CException $e) {
            throw $e;
        }
    }

    /**
     * Get the content for the view in a String.
     * @return the generated content.
     */
    public function fetch() {
        try {
            extract($this->_someVars);
            ob_start();

            if (is_file($this->_sPath . $this->_sView)) {
                include($this->_sPath . $this->_sView);
            } else {
                echo "View $this->_sPath $this->_sView Not Found";
            }

            $sContent = ob_get_contents();
            ob_end_clean();
        }
        catch(CException $e) {
            throw $e;
        }
        return $sContent;
    }

    public function element($p_sElement, $p_someVars = null)
    {
        try {
            if( $p_someVars != null )
                extract($p_someVars);
            
            extract($this->_someVars);
            ob_start();

            $sPath = C_ELEMENTS.$p_sElement.'.php';
            if (is_file($sPath)) {
                include($sPath);
            } else {
                echo "Element Not Found";
            }

            $sContent = ob_get_contents();
            ob_end_clean();

        }
        catch(CException $e) {
            throw $e;
        }
        echo $sContent;
    }

    public function showParentUrl($p_sController = null)
    {
        try {
            $sController = $this->_sParentController;

            if($p_sController != null)
                $sController = $p_sController;

            if($this->_sLanguageDefault)
                return __($sController,$this->_sLanguage,true)."/";
            else
                return "$this->_sLanguage/".__($sController,$this->_sLanguage,true)."/";
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function showParentUrlLanguage($p_sLanguage, $p_sController = null) {
        try {
            $sController = $this->_sParentController;

            if($p_sController != null)
                  $sController = $p_sController;

            $sPage = "";
            if (isset($this->_someVars["page"])) {
                //if you use 5.3.0 you can use stristr function with before_needle parameter
                $sPage = $this->_someVars["page"];
                $iPosition = stripos($sPage,".");
                $sPage = substr($sPage, 0, $iPosition);

            }

            return "$p_sLanguage/".__($sController,$p_sLanguage,true)."/".$sPage;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function issetField( $p_sFieldName , $p_sTable = null )
    {
        $sReturn = "";
        try {
            if(isset($this->_someVars['someData']))
            {
                if(isset($this->_someVars['someData'][$this->_sParentController][$p_sFieldName]))
                {
                    $sReturn = $this->_someVars['someData'][$this->_sParentController][$p_sFieldName];
                }
                else
                {
                    if( !is_null($p_sTable) && isset($this->_someVars['someData'][$p_sTable][$p_sFieldName]))
                    {
                        $sReturn = $this->_someVars['someData'][$p_sTable][$p_sFieldName];
                    }
                    else
                    {
                        $sReturn = "";
                    }
                }
            }
            else
            {
                $sReturn = "";
            }
        }
        catch(CException $e) {
            throw $e;
        }
        return $sReturn;
    }

    public function issetSomeData()
    {
        $sReturn = false;
        try {
            if(isset($this->_someVars['someData']))
            {
                $sReturn = true;
            }
        }
        catch(CException $e) {
            throw $e;
        }
        return $sReturn;
    }
    
    /**
     * GETTERS
     */

    /**
     * Get the value of a template variable.
     * @param $p_sName the name of the view that we want to get
     * @return the value of the passed parameter variable
     */
    public function get( $p_sName , $p_sDefault = "" ) {
        try {
            if( isset( $this->_someVars[ $p_sName ] ) )
                return $this->_someVars[ $p_sName ];
            else
                return $p_sDefault;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function get_sLanguageDefault() {
        try {
            return $this->_sLanguageDefault;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function get_sLanguage() {
        try {
            return $this->_sLanguage;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function get_someData($p_sColumn = "") {
        try {
            if(isset($this->_someVars['someData']))
            {
                if(isset($this->_someVars['someData'][$this->_sParentController][$p_sColumn]))
                    return $this->_someVars['someData'][$this->_sParentController][$p_sColumn];
                else
                    return "";
            }
            else
                return "";
        }
        catch(CException $e) {
            throw $e;
        }

        return "";
    }

    public function get_sView()
    {
        try{
            $sView = str_replace('.php', '', $this->_sView);
            $bView = false;
            $sReturnPath = $sView;

            foreach ( $_GET as $sParameter )
            {
                if($bView)
                {
                    if ( preg_match( "/sort/" , $sParameter ) == 1 || preg_match( "/direction/" , $sParameter ) == 1 || preg_match( "/page/" , $sParameter ) == 1)
                        break;
                    else
                        $sReturnPath .= "/$sParameter";
                }

                if ($sParameter == $sView)
                    $bView = true;
            }
            return $sReturnPath;
    //        return str_replace('.php', '', $this->_sView);
        }catch(CException $e){
            throw $e;
        }
    }

    public function get_sParentController() {
        return $this->_sParentController;
    }

    /**
     * SETTERS
     */

    /**
     * Set the value of a template variable.
     * @param $p_sName the name of the view that we want to get
     * @return the value of the passed parameter variable
     */
    public function set( $p_sName , $p_sValue ) {
        try {
            $this->_someVars[$p_sName] = $p_sValue;;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function set_sLanguageDefault($p_sLanguageDefault) {
        try {
            $this->_sLanguageDefault = $p_sLanguageDefault;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function set_sParentController($p_sParentController) {
        $this->_sParentController = $p_sParentController;
    }

    /**
     * this function assign a view file.
     *
     * @param String $p_sFile name of the file
     */
    public function set_sView($p_sFile)
    {
        try {
            $this->_sView = $p_sFile;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    /**
     * this function assign a view file.
     *
     * @param String $p_sFile name of the file
     */
    public function setPath($p_sPath)
    {
        try {
            $this->_sPath = $p_sPath;
        }
        catch(CException $e) {
            throw $e;
        }
    }

    /**
     * this function assign a parent file.
     *
     * @param String $p_sParentUrl name of the file
     */
    public function set_sParentUrl($p_sParentController , $p_sLanguage = 'es' , $p_sLanguageDefault = true )
    {
        try {
            $this->_sParentController = $p_sParentController;
            $this->set_sLanguage($p_sLanguage);
            $this->set_sLanguageDefault($p_sLanguageDefault);
        }
        catch(CException $e) {
            throw $e;
        }
    }

    public function set_sLanguage($p_sLanguage) {
        try {
            $this->_sLanguage = $p_sLanguage;
        }
        catch(CException $e) {
            throw $e;
        }
    }
}
?>
